@@ -11,7 +11,6 @@ use File::Spec;
 
 use lib '.';
 use build::setup;
-use build::auto;
 use build::probe;
 
 # This allows us to run on ancient perls.
@@ -41,7 +40,7 @@ my @args = @ARGV;
 
 GetOptions(\%args, qw(
     help|?
-    debug:s optimize:s instrument! coverage
+    debug:s optimize:s coverage
     os=s shell=s toolchain=s compiler=s
     ar=s cc=s ld=s make=s has-sha has-libuv
     static has-libtommath has-libatomic_ops
@@ -49,12 +48,11 @@ GetOptions(\%args, qw(
     build=s host=s big-endian jit! enable-jit
     prefix=s bindir=s libdir=s mastdir=s
     relocatable make-install asan ubsan tsan
-    valgrind telemeh dtrace show-autovect git-cache-dir=s
-    show-autovect-failed:s),
+    valgrind telemeh! dtrace show-autovect git-cache-dir=s
+    show-autovect-failed:s mimalloc! c11-atomics!),
 
     'no-optimize|nooptimize' => sub { $args{optimize} = 0 },
     'no-debug|nodebug' => sub { $args{debug} = 0 },
-    'no-telemeh|notelemeh' => sub { $args{telemeh} = 0 }
 ) or die "See --help for further information\n";
 
 
@@ -92,7 +90,7 @@ if ( $args{relocatable} && ($^O eq 'aix' || $^O eq 'openbsd') ) {
     ".\n    Leave off the --relocatable flag to do a non-relocatable build.");
 }
 
-for (qw(coverage instrument static big-endian has-libtommath has-sha has-libuv
+for (qw(coverage static big-endian has-libtommath has-sha has-libuv
         has-libatomic_ops asan ubsan tsan valgrind dtrace show-vec)) {
     $args{$_} = 0 unless defined $args{$_};
 }
@@ -217,12 +215,14 @@ if ($config{cc} eq 'gcc' && !$config{can_specific_werror}) {
 
 # Disable RETGUARD on OpenBSD, since it breaks the legojit.
 if ($^O eq 'openbsd' && $args{jit} && $config{cc} eq 'clang') {
-	$config{ccmiscflags} .= ' -fno-ret-protector';
+    $config{ccmiscflags} .= ' -fno-ret-protector';
 }
 
 # Set the remaining ldmiscflags. Do this after probing for gcc -Werror probe to not miss that change for the linker.
 $config{ldmiscflags}  = $config{ccmiscflags} unless defined $config{ldmiscflags};
 
+# Include paths that NQP/Rakudo are going to need in their build.
+my @hllincludes = qw( moar );
 
 if ($args{'has-sha'}) {
     $config{shaincludedir} = '/usr/include/sha';
@@ -231,7 +231,7 @@ if ($args{'has-sha'}) {
 }
 else { $config{shaincludedir} = '3rdparty/sha1' }
 
-# After upgrading from libuv from 0.11.18 to 0.11.29 we see very weird erros
+# After upgrading from libuv from 0.11.18 to 0.11.29 we see very weird errors
 # when the old libuv files are still around. Running a `make realclean` in
 # case we spot an old file and the Makefile is already there.
 if (-e '3rdparty/libuv/src/unix/threadpool' . $defaults{obj}
@@ -269,9 +269,15 @@ else {
                         . "\t\$(MKPATH) \"\$(DESTDIR)\$(PREFIX)/include/libuv/uv\"\n"
                         . "\t\$(CP) 3rdparty/libuv/include/*.h \"\$(DESTDIR)\$(PREFIX)/include/libuv\"\n"
                         . "\t\$(CP) 3rdparty/libuv/include/uv/*.h \"\$(DESTDIR)\$(PREFIX)/include/libuv/uv\"\n";
+    push @hllincludes, 'libuv';
 }
 
-if ($args{'has-libatomic_ops'}) {
+$config{use_c11_atomics} = $args{'c11-atomics'} ? 1 : 0;
+
+if ($config{use_c11_atomics}) {
+    $defaults{-thirdparty}->{lao} = undef;
+}
+elsif ($args{'has-libatomic_ops'}) {
     $defaults{-thirdparty}->{lao} = undef;
     unshift @{$config{usrlibs}}, 'atomic_ops';
     setup_native_library('atomic_ops') if $config{pkgconfig_works};
@@ -298,6 +304,7 @@ else {
                         . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/loadstore/*.h \"$lao/atomic_ops/sysdeps/loadstore\"\n"
                         . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/msftc/*.h \"$lao/atomic_ops/sysdeps/msftc\"\n"
                         . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/sunc/*.h \"$lao/atomic_ops/sysdeps/sunc\"\n";
+    push @hllincludes, 'libatomic_ops';
 }
 
 if ($args{'has-libtommath'}) {
@@ -316,6 +323,7 @@ else {
     $config{moar_cincludes} .= ' ' . $defaults{ccinc} . '3rdparty/libtommath';
     $config{install}   .= "\t\$(MKPATH) \"\$(DESTDIR)\$(PREFIX)/include/libtommath\"\n"
                         . "\t\$(CP) 3rdparty/libtommath/*.h \"\$(DESTDIR)\$(PREFIX)/include/libtommath\"\n";
+    push @hllincludes, 'libtommath';
 }
 
 if ($args{'has-libffi'}) {
@@ -363,9 +371,12 @@ else {
                         . "\t\$(CP) 3rdparty/dyncall/dynload/*.h \"\$(DESTDIR)\$(PREFIX)/include/dyncall\"\n"
                         . "\t\$(CP) 3rdparty/dyncall/dyncall/*.h \"\$(DESTDIR)\$(PREFIX)/include/dyncall\"\n"
                         . "\t\$(CP) 3rdparty/dyncall/dyncallback/*.h \"\$(DESTDIR)\$(PREFIX)/include/dyncall\"\n";
+    push @hllincludes, 'dyncall';
 }
 
-if ($config{pkgconfig_works} && has_native_library('libzstd')) {
+# The ZSTD_CStream API is only exposed starting at version 1.0.0
+# Ubuntu 16.04 packages version 0.5.1, so an "exists" check is not good enough.
+if ($config{pkgconfig_works} && has_native_library('libzstd', '1.0.0')) {
     setup_native_library('libzstd');
     $config{heapsnapformat} = 3;
 }
@@ -394,11 +405,11 @@ $config{lddebugflags} = sprintf $config{lddebugflags}, defined_or $args{debug},
 
 # generate CFLAGS
 my @cflags;
+push @cflags, '-std=c99' if $defaults{os} eq 'mingw32';
 push @cflags, $config{ccmiscflags};
 push @cflags, $config{ccoptiflags}  if $args{optimize};
 push @cflags, $config{ccdebugflags} if $args{debug};
-push @cflags, $config{ccinstflags}  if $args{instrument};
-push @cflags, $config{ld_covflags}  if $args{coverage};
+push @cflags, $config{cc_covflags}  if $args{coverage};
 push @cflags, $config{ccwarnflags};
 push @cflags, $config{ccdefflags};
 push @cflags, $config{ccshared}     unless $args{static};
@@ -432,13 +443,23 @@ push @cflags, '-DWORDS_BIGENDIAN' if $config{be}; # 3rdparty/sha1 needs it and i
 push @cflags, '-DMVM_HEAPSNAPSHOT_FORMAT=' . $config{heapsnapformat};
 push @cflags, $ENV{CFLAGS} if $ENV{CFLAGS};
 push @cflags, $ENV{CPPFLAGS} if $ENV{CPPFLAGS};
+
+# Define _GNU_SOURCE for libuv to quell warnings with gcc.
+# According to the libuv developers, they define _GNU_SOURCE for
+# Linux use. Our code was showing warnings from libuv
+# because we did not. Adding -D_GNU_SOURCE conditionally
+# to our build then resulted in redefinition warnings in
+# our code for three of our files that already have that definition.
+# The fix was to bracket those definitions with
+# #ifdef _GNU_SOURCE/#endif.
+push @cflags, '-D_GNU_SOURCE' unless $args{'has-libuv'};
+
 $config{cflags} = join ' ', uniq(@cflags);
 
 # generate LDFLAGS
 my @ldflags = ($config{ldmiscflags});
 push @ldflags, $config{ldoptiflags}  if $args{optimize};
 push @ldflags, $config{lddebugflags} if $args{debug};
-push @ldflags, $config{ldinstflags}  if $args{instrument};
 push @ldflags, $config{ld_covflags}  if $args{coverage};
 if (not $args{static} and $config{prefix} ne '/usr') {
     push @ldflags, $config{ldrpath_relocatable} if  $args{relocatable};
@@ -487,21 +508,83 @@ $config{auxclean} = @auxfiles ? '$(RM) ' . join ' ', @auxfiles : '@:';
 
 print "OK\n\n";
 
-if ($config{crossconf}) {
-    build::auto::detect_cross(\%config, \%defaults);
-    build::probe::static_inline_cross(\%config, \%defaults);
-    build::probe::thread_local_cross(\%config, \%defaults);
-    build::probe::unaligned_access_cross(\%config, \%defaults);
-    build::probe::ptr_size_cross(\%config, \%defaults);
-}
-else {
-    build::auto::detect_native(\%config, \%defaults);
-    build::probe::static_inline_native(\%config, \%defaults);
-    build::probe::thread_local_native(\%config, \%defaults);
-    build::probe::unaligned_access(\%config, \%defaults);
-    build::probe::ptr_size_native(\%config, \%defaults);
+unless ($config{crossconf}) {
+    # detect x64 on Windows so we can build the correct dyncall version
+    if ($config{cc} eq 'cl') {
+        print dots('    auto-detecting x64 toolchain');
+        my $msg = `cl 2>&1`;
+        if (defined $msg) {
+            if ($msg =~ /x64/) {
+                print "YES\n";
+                $defaults{-thirdparty}->{dc}->{rule} =
+                    'cd 3rdparty/dyncall && configure.bat /target-x64 && $(MAKE) -f Nmakefile';
+            }
+            else { print "NO\n" }
+        }
+        else {
+            softfail("could not run 'cl'");
+            print dots('    assuming x86'), "OK\n";
+        }
+    }
+    elsif ($defaults{os} eq 'mingw32' && $defaults{-toolchain} eq 'gnu') {
+        print dots('    auto-detecting x64 toolchain');
+        my $cc = $config{cc};
+        my $msg = `$cc -dumpmachine 2>&1`;
+        if (defined $msg) {
+            if ($msg =~ /x86_64/) {
+                print "YES\n";
+
+                $defaults{-thirdparty}->{dc}->{rule} =
+                    'cd 3rdparty/dyncall && ./configure.bat /target-x64 /tool-gcc && $(MAKE) COMPILE.C=$$(COMPILE.c) -f Makefile.embedded mingw32';
+            }
+            else { print "NO\n" }
+        }
+        else {
+            softfail("could not run 'cl'");
+            print dots('    assuming x86'), "OK\n";
+        }
+    }
 }
 
+build::probe::static_inline(\%config, \%defaults);
+build::probe::thread_local(\%config, \%defaults);
+build::probe::substandard_pow(\%config, \%defaults);
+build::probe::substandard_log(\%config, \%defaults);
+build::probe::substandard_trig(\%config, \%defaults);
+build::probe::has_isinf_and_isnan(\%config, \%defaults);
+build::probe::unaligned_access(\%config, \%defaults);
+build::probe::ptr_size(\%config, \%defaults);
+build::probe::stdatomic(\%config, \%defaults);
+
+$config{use_mimalloc} = $args{mimalloc};
+if (!defined $config{use_mimalloc}) {
+    if ($config{has_stdatomic}) {
+        print "Defaulting to mimalloc because you have <stdatomic.h>\n";
+        $config{use_mimalloc} = 1;
+    }
+    elsif ($config{cc} eq 'cl') {
+        print "Defaulting to mimalloc because you are using MSVC\n";
+        $config{use_mimalloc} = 1;
+    }
+    else {
+        print "Defaulting to libc malloc because <stdatomic.h> was not found.\n";
+        $config{use_mimalloc} = 0;
+    }
+}
+
+if ($config{use_mimalloc}) {
+    $config{moar_cincludes} .= ' ' . $defaults{ccinc} . '3rdparty/mimalloc/include'
+                             . ' ' . $defaults{ccinc} . '3rdparty/mimalloc/src';
+    $config{install}   .= "\t\$(MKPATH) \"\$(DESTDIR)\$(PREFIX)/include/mimalloc\"\n"
+                       . "\t\$(CP) 3rdparty/mimalloc/include/*.h \"\$(DESTDIR)\$(PREFIX)/include/mimalloc\"\n";
+    push @hllincludes, 'mimalloc';
+    $config{mimalloc_include} = '@ccincsystem@3rdparty/mimalloc';
+    $config{mimalloc_object} = '3rdparty/mimalloc/src/static@obj@';
+}
+else {
+    $config{mimalloc_include} = "";
+    $config{mimalloc_object} = "";
+}
 
 my $archname = $Config{archname};
 if ($args{'jit'}) {
@@ -547,10 +630,8 @@ unless (defined $config{jit_obj}) {
     $config{dasm_flags}   = '';
 }
 
-
-if ($config{cc} eq 'cl') {
-    $config{install}   .= "\t\$(MKPATH) \"\$(DESTDIR)\$(PREFIX)/include/msinttypes\"\n"
-                        . "\t\$(CP) 3rdparty/msinttypes/*.h \"\$(DESTDIR)\$(PREFIX)/include/msinttypes\"\n";
+if ($^O eq 'aix' && $config{ptr_size} == 4) {
+    $config{ldflags} = join(',', $config{ldflags}, '-bmaxdata:0x80000000');
 }
 
 build::probe::C_type_bool(\%config, \%defaults);
@@ -558,11 +639,6 @@ build::probe::computed_goto(\%config, \%defaults);
 build::probe::pthread_yield(\%config, \%defaults);
 build::probe::pthread_setname_np(\%config, \%defaults);
 build::probe::check_fn_malloc_trim(\%config, \%defaults);
-if ($^O eq 'aix') {
-    build::probe::numbits(\%config, \%defaults);
-    $config{ldflags} = join(',', $config{ldflags}, '-bmaxdata:0x80000000')
-        if $config{arch_bits} == 32;
-}
 build::probe::rdtscp(\%config, \%defaults);
 
 my $order = $config{be} ? 'big endian' : 'little endian';
@@ -646,6 +722,10 @@ my $thirdpartylibs = join "\n" . ' ' x 12, sort @thirdpartylibs;
 
 print "OK\n";
 
+# the parser for config values used in the NQP/Rakudo build doesn't understand arrays,
+# so just join into a single string
+$config{hllincludes} = join " ", @hllincludes;
+
 write_backend_config();
 
 # dump 3rdparty libs we need to build
@@ -709,8 +789,20 @@ sub setup_native {
     print dots("Configuring native build environment");
     print "\n";
 
-    $os = build::probe::win32_compiler_toolchain(\%config, \%defaults)
-        if $os eq 'MSWin32';
+    if ($os eq 'MSWin32') {
+        my $has_nmake = 0 == system('nmake /? >NUL 2>&1');
+        my $has_cl    = `cl 2>&1` =~ /Microsoft Corporation/;
+        my $has_gmake = 0 == system('gmake --version >NUL 2>&1');
+        my $has_gcc   = 0 == system('gcc --version >NUL 2>&1');
+        if ($has_nmake && $has_cl) {
+            $os = 'win32';
+        }
+        elsif ($has_gmake && $has_gcc) {
+            $os = 'mingw32';
+        } else {
+            $os = "";
+        }
+    }
 
     if (!exists $::SYSTEMS{$os}) {
         softfail("unknown OS '$os'");
@@ -836,7 +928,18 @@ sub configure {
                 join(', ', sort keys %config));
         }
 
-        $template =~ s/@\Q$key\E@/$config{$key}/;
+        my $val = $config{$key};
+        if ($template =~ /'[^']*@\Q$key\E[^']*'/) {
+            # escape \ and '
+            $val =~ s/\\/\\\\/g;
+            $val =~ s/\'/\\\'/g;
+        } elsif ($template =~ /"[^"]*@\Q$key\E[^"]*"/) {
+            # escape \ and "
+            $val =~ s/\\/\\\\/g;
+            $val =~ s/\"/\\\"/g;
+        }
+
+        $template =~ s/@\Q$key\E@/$val/;
     }
 
     return $template;
@@ -958,14 +1061,11 @@ sub setup_native_library {
 }
 
 sub has_native_library {
-    my $library = shift;
-    my $result_exists = `$config{pkgconfig} --exists $library`;
-    if ($? == 0) {
-        return 1;
-    }
-    else {
-        return 0;
+    my ($library, $version) = @_;
+    if (defined $version) {
+        return !system "$config{pkgconfig} --atleast-version=$version $library";
     }
+    return !system "$config{pkgconfig} --exists $library"
 }
 
 __END__
@@ -977,7 +1077,7 @@ __END__
     ./Configure.pl [--os <os>] [--shell <shell>]
                    [--toolchain <toolchain>] [--compiler <compiler>]
                    [--ar <ar>] [--cc <cc>] [--ld <ld>] [--make <make>]
-                   [--debug] [--optimize] [--instrument]
+                   [--debug] [--optimize]
                    [--static] [--prefix <path>] [--relocatable]
                    [--has-libtommath] [--has-sha] [--has-libuv]
                    [--has-libatomic_ops]
@@ -986,7 +1086,7 @@ __END__
 
     ./Configure.pl --build <build-triple> --host <host-triple>
                    [--ar <ar>] [--cc <cc>] [--ld <ld>] [--make <make>]
-                   [--debug] [--optimize] [--instrument]
+                   [--debug] [--optimize]
                    [--static] [--prefix <path>] [--relocatable]
                    [--big-endian] [--make-install]
 
@@ -1020,12 +1120,26 @@ default.
 Toggle optimization and debug flags during compile and link. If nothing
 is specified the default is to optimize.
 
-=item --instrument
+=item --mimalloc
 
-=item --no-instrument
+=item --no-mimalloc
 
-Toggle extra instrumentation flags during compile and link; for example,
-turns on Address Sanitizer when compiling with C<clang>.  Defaults to off.
+Control whether we build with mimalloc or use the C library's malloc.
+
+mimalloc requires either MSVC or working C11 C<< <stdatomic> >>. We probe for
+this, and if found or using MSVC we default to use mimalloc. Otherwise we
+default to the C library's malloc. Specify C<--no-mimalloc> to force use of the
+C library's malloc always. Specify C<--mimalloc> to force use of mimalloc, even
+if the probing thinks that it won't build.
+
+=item --c11-atomics
+
+=item --no-c11-atomics
+
+Use C11 atomics instead of libatomic_ops for atomic operations. The default
+is currently C<--no-c11-atomics> - ie use libatomic_ops. If you set
+C<--c11-atomics> and your compiler does not support C11 atomics, your build
+will fail.
 
 =item --os <os>
 
