@@ -35,6 +35,10 @@ static void code_pair_fetch_i(MVMThreadContext *tc, MVMObject *cont, MVMRegister
     code_pair_fetch_internal(tc, cont, res, MVM_RETURN_INT);
 }
 
+static void code_pair_fetch_u(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    code_pair_fetch_internal(tc, cont, res, MVM_RETURN_INT);
+}
+
 static void code_pair_fetch_n(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
     code_pair_fetch_internal(tc, cont, res, MVM_RETURN_NUM);
 }
@@ -105,10 +109,12 @@ static const MVMContainerSpec code_pair_spec = {
     "code_pair",
     code_pair_fetch,
     code_pair_fetch_i,
+    code_pair_fetch_u,
     code_pair_fetch_n,
     code_pair_fetch_s,
     code_pair_store,
     code_pair_store_i,
+    code_pair_store_i, /* FIXME need a code_pair_store_u but lacking tests showing this need */
     code_pair_store_n,
     code_pair_store_s,
     code_pair_store,
@@ -120,8 +126,8 @@ static const MVMContainerSpec code_pair_spec = {
     NULL,
     code_pair_can_store,
     NULL, /* cas */
-    NULL, /* atomic_load */
-    NULL, /* atomic_store */
+    NULL, /* load_atomic */
+    NULL, /* store_atomic */
     0
 };
 
@@ -178,7 +184,7 @@ typedef struct {
     MVMCode *store;
     MVMCode *store_unchecked;
     MVMCode *cas;
-    MVMCode *atomic_store;
+    MVMCode *store_atomic;
 
     /* Retained for serialization purposes only. */
     MVMObject *attrs_class;
@@ -212,6 +218,10 @@ static void value_desc_cont_fetch_i(MVMThreadContext *tc, MVMObject *cont, MVMRe
     res->i64 = MVM_repr_get_int(tc, read_container_value(cont));
 }
 
+static void value_desc_cont_fetch_u(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    res->u64 = MVM_repr_get_uint(tc, read_container_value(cont));
+}
+
 static void value_desc_cont_fetch_n(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
     res->n64 = MVM_repr_get_num(tc, read_container_value(cont));
 }
@@ -267,7 +277,7 @@ static void value_desc_cont_gc_mark_data(MVMThreadContext *tc, MVMSTable *st, MV
     MVM_gc_worklist_add(tc, worklist, &data->store);
     MVM_gc_worklist_add(tc, worklist, &data->store_unchecked);
     MVM_gc_worklist_add(tc, worklist, &data->cas);
-    MVM_gc_worklist_add(tc, worklist, &data->atomic_store);
+    MVM_gc_worklist_add(tc, worklist, &data->store_atomic);
     MVM_gc_worklist_add(tc, worklist, &data->attrs_class);
     MVM_gc_worklist_add(tc, worklist, &data->value_attr);
     MVM_gc_worklist_add(tc, worklist, &data->descriptor_attr);
@@ -282,7 +292,7 @@ static void value_desc_cont_serialize(MVMThreadContext *tc, MVMSTable *st, MVMSe
     MVM_serialization_write_ref(tc, writer, (MVMObject *)data->store);
     MVM_serialization_write_ref(tc, writer, (MVMObject *)data->store_unchecked);
     MVM_serialization_write_ref(tc, writer, (MVMObject *)data->cas);
-    MVM_serialization_write_ref(tc, writer, (MVMObject *)data->atomic_store);
+    MVM_serialization_write_ref(tc, writer, (MVMObject *)data->store_atomic);
     MVM_serialization_write_ref(tc, writer, data->attrs_class);
     MVM_serialization_write_str(tc, writer, data->value_attr);
     MVM_serialization_write_str(tc, writer, data->descriptor_attr);
@@ -293,7 +303,7 @@ static void value_desc_cont_deserialize(MVMThreadContext *tc, MVMSTable *st, MVM
     MVM_ASSIGN_REF(tc, &(st->header), data->store, MVM_serialization_read_ref(tc, reader));
     MVM_ASSIGN_REF(tc, &(st->header), data->store_unchecked, MVM_serialization_read_ref(tc, reader));
     MVM_ASSIGN_REF(tc, &(st->header), data->cas, MVM_serialization_read_ref(tc, reader));
-    MVM_ASSIGN_REF(tc, &(st->header), data->atomic_store, MVM_serialization_read_ref(tc, reader));
+    MVM_ASSIGN_REF(tc, &(st->header), data->store_atomic, MVM_serialization_read_ref(tc, reader));
     MVM_ASSIGN_REF(tc, &(st->header), data->attrs_class, MVM_serialization_read_ref(tc, reader));
     MVM_ASSIGN_REF(tc, &(st->header), data->value_attr, MVM_serialization_read_str(tc, reader));
     MVM_ASSIGN_REF(tc, &(st->header), data->descriptor_attr, MVM_serialization_read_str(tc, reader));
@@ -343,23 +353,25 @@ static MVMObject * value_desc_cont_atomic_load(MVMThreadContext *tc, MVMObject *
     return value ? value : tc->instance->VMNull;
 }
 
-void value_desc_cont_atomic_store(MVMThreadContext *tc, MVMObject *cont, MVMObject *value) {
+static void value_desc_cont_atomic_store(MVMThreadContext *tc, MVMObject *cont, MVMObject *value) {
     MVMValueDescContainer *data = (MVMValueDescContainer *)STABLE(cont)->container_data;
     MVMCallStackArgsFromC *args_record = MVM_callstack_allocate_args_from_c(tc,
             MVM_callsite_get_common(tc, MVM_CALLSITE_ID_OBJ_OBJ));
     args_record->args.source[0].o = cont;
     args_record->args.source[1].o = value;
-    MVM_frame_dispatch_from_c(tc, data->atomic_store, args_record, NULL, MVM_RETURN_VOID);
+    MVM_frame_dispatch_from_c(tc, data->store_atomic, args_record, NULL, MVM_RETURN_VOID);
 }
 
 static const MVMContainerSpec value_desc_cont_spec = {
     "value_desc_cont",
     value_desc_cont_fetch,
     value_desc_cont_fetch_i,
+    value_desc_cont_fetch_u,
     value_desc_cont_fetch_n,
     value_desc_cont_fetch_s,
     value_desc_cont_store,
     value_desc_cont_store_i,
+    value_desc_cont_store_i, /* FIXME need a value_desc_cont_store_u but lacking tests showing this need */
     value_desc_cont_store_n,
     value_desc_cont_store_s,
     value_desc_cont_store_unchecked,
@@ -412,7 +424,7 @@ static void value_desc_cont_configure_container_spec(MVMThreadContext *tc, MVMST
         value = grab_one_value(tc, config, "atomic_store");
         if (!MVM_code_iscode(tc, value))
             MVM_exception_throw_adhoc(tc, "Container spec must be configured with a code handle");
-        MVM_ASSIGN_REF(tc, &(st->header), data->atomic_store, value);
+        MVM_ASSIGN_REF(tc, &(st->header), data->store_atomic, value);
         value = grab_one_value(tc, config, "attrs_class");
         MVM_ASSIGN_REF(tc, &(st->header), data->attrs_class, value);
         value = grab_one_value(tc, config, "value_attr");
@@ -455,6 +467,28 @@ static void native_ref_fetch_i(MVMThreadContext *tc, MVMObject *cont, MVMRegiste
     }
 }
 
+static void native_ref_fetch_u(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
+    MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
+    if (repr_data->primitive_type != MVM_STORAGE_SPEC_BP_UINT64)
+        MVM_exception_throw_adhoc(tc, "This container does not reference a native unsigned integer");
+    switch (repr_data->ref_kind) {
+        case MVM_NATIVEREF_LEX:
+            res->u64 = MVM_nativeref_read_lex_i(tc, cont); /* covers unsigned as well */
+            break;
+        case MVM_NATIVEREF_ATTRIBUTE:
+            res->u64 = MVM_nativeref_read_attribute_u(tc, cont);
+            break;
+        case MVM_NATIVEREF_POSITIONAL:
+            res->u64 = MVM_nativeref_read_positional_u(tc, cont);
+            break;
+        case MVM_NATIVEREF_MULTIDIM:
+            res->u64 = MVM_nativeref_read_multidim_u(tc, cont);
+            break;
+        default:
+            MVM_exception_throw_adhoc(tc, "Unknown native int reference kind");
+    }
+}
+
 static void native_ref_fetch_n(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
     MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
     if (repr_data->primitive_type != MVM_STORAGE_SPEC_BP_NUM)
@@ -507,8 +541,15 @@ static void native_ref_fetch(MVMThreadContext *tc, MVMObject *cont, MVMRegister
         hll = MVM_hll_current(tc);
     switch (repr_data->primitive_type) {
         case MVM_STORAGE_SPEC_BP_INT:
-            native_ref_fetch_i(tc, cont, &tmp);
-            res->o = MVM_repr_box_int(tc, hll->int_box_type, tmp.i64);
+        case MVM_STORAGE_SPEC_BP_UINT64:
+            if (repr_data->is_unsigned) {
+                native_ref_fetch_u(tc, cont, &tmp);
+                res->o = MVM_repr_box_uint(tc, hll->int_box_type, tmp.u64);
+            }
+            else {
+                native_ref_fetch_i(tc, cont, &tmp);
+                res->o = MVM_repr_box_int(tc, hll->int_box_type, tmp.i64);
+            }
             break;
         case MVM_STORAGE_SPEC_BP_NUM:
             native_ref_fetch_n(tc, cont, &tmp);
@@ -545,6 +586,28 @@ static void native_ref_store_i(MVMThreadContext *tc, MVMObject *cont, MVMint64 v
     }
 }
 
+static void native_ref_store_u(MVMThreadContext *tc, MVMObject *cont, MVMuint64 value) {
+    MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
+    if (repr_data->primitive_type != MVM_STORAGE_SPEC_BP_UINT64)
+        MVM_exception_throw_adhoc(tc, "This container does not reference a native integer");
+    switch (repr_data->ref_kind) {
+        case MVM_NATIVEREF_LEX:
+            MVM_nativeref_write_lex_i(tc, cont, value); /* FIXME need a MVM_nativeref_write_lex_u */
+            break;
+        case MVM_NATIVEREF_ATTRIBUTE:
+            MVM_nativeref_write_attribute_u(tc, cont, value);
+            break;
+        case MVM_NATIVEREF_POSITIONAL:
+            MVM_nativeref_write_positional_u(tc, cont, value);
+            break;
+        case MVM_NATIVEREF_MULTIDIM:
+            MVM_nativeref_write_multidim_u(tc, cont, value);
+            break;
+        default:
+            MVM_exception_throw_adhoc(tc, "Unknown native int reference kind");
+    }
+}
+
 static void native_ref_store_n(MVMThreadContext *tc, MVMObject *cont, MVMnum64 value) {
     MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
     if (repr_data->primitive_type != MVM_STORAGE_SPEC_BP_NUM)
@@ -592,8 +655,12 @@ static void native_ref_store_s(MVMThreadContext *tc, MVMObject *cont, MVMString
 static void native_ref_store(MVMThreadContext *tc, MVMObject *cont, MVMObject *obj) {
     MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
     switch (repr_data->primitive_type) {
+        case MVM_STORAGE_SPEC_BP_UINT64:
         case MVM_STORAGE_SPEC_BP_INT:
-            native_ref_store_i(tc, cont, MVM_repr_get_int(tc, obj));
+            if (repr_data->is_unsigned)
+                native_ref_store_u(tc, cont, MVM_repr_get_uint(tc, obj));
+            else
+                native_ref_store_i(tc, cont, MVM_repr_get_int(tc, obj));
             break;
         case MVM_STORAGE_SPEC_BP_NUM:
             native_ref_store_n(tc, cont, MVM_repr_get_num(tc, obj));
@@ -622,10 +689,12 @@ static const MVMContainerSpec native_ref_spec = {
     "native_ref",
     native_ref_fetch,
     native_ref_fetch_i,
+    native_ref_fetch_u,
     native_ref_fetch_n,
     native_ref_fetch_s,
     native_ref_store,
     native_ref_store_i,
+    native_ref_store_u,
     native_ref_store_n,
     native_ref_store_s,
     native_ref_store,
@@ -637,8 +706,8 @@ static const MVMContainerSpec native_ref_spec = {
     NULL,
     native_ref_can_store,
     NULL, /* cas */
-    NULL, /* atomic_load */
-    NULL, /* atomic_store */
+    NULL, /* load_atomic */
+    NULL, /* store_atomic */
     1
 };
 
@@ -786,7 +855,24 @@ static MVMint64 get_container_primitive(MVMThreadContext *tc, MVMObject *cont) {
     return MVM_STORAGE_SPEC_BP_NONE;
 }
 MVMint64 MVM_6model_container_iscont_i(MVMThreadContext *tc, MVMObject *cont) {
-    return get_container_primitive(tc, cont) == MVM_STORAGE_SPEC_BP_INT;
+    if (cont && IS_CONCRETE(cont)) {
+        const MVMContainerSpec *cs = STABLE(cont)->container_spec;
+        if (cs == &native_ref_spec && REPR(cont)->ID == MVM_REPR_ID_NativeRef) {
+            MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
+            return repr_data->primitive_type == MVM_STORAGE_SPEC_BP_INT && !repr_data->is_unsigned;
+        }
+    }
+    return 0;
+}
+MVMint64 MVM_6model_container_iscont_u(MVMThreadContext *tc, MVMObject *cont) {
+    if (cont && IS_CONCRETE(cont)) {
+        const MVMContainerSpec *cs = STABLE(cont)->container_spec;
+        if (cs == &native_ref_spec && REPR(cont)->ID == MVM_REPR_ID_NativeRef) {
+            MVMNativeRefREPRData *repr_data = (MVMNativeRefREPRData *)STABLE(cont)->REPR_data;
+            return repr_data->primitive_type == MVM_STORAGE_SPEC_BP_UINT64 && repr_data->is_unsigned;
+        }
+    }
+    return 0;
 }
 MVMint64 MVM_6model_container_iscont_n(MVMThreadContext *tc, MVMObject *cont) {
     return get_container_primitive(tc, cont) == MVM_STORAGE_SPEC_BP_NUM;
@@ -825,13 +911,12 @@ void MVM_6model_container_decont_s(MVMThreadContext *tc, MVMObject *cont, MVMReg
         res->s = MVM_repr_get_str(tc, cont);
 }
 
-/* If it's a container, do a fetch_i. Otherwise, try to unbox the received
+/* If it's a container, do a fetch_u. Otherwise, try to unbox the received
  * value as a native unsigned integer. */
 void MVM_6model_container_decont_u(MVMThreadContext *tc, MVMObject *cont, MVMRegister *res) {
     const MVMContainerSpec *cs = STABLE(cont)->container_spec;
     if (cs && IS_CONCRETE(cont))
-        /* XXX We need a fetch_u at some point. */
-        cs->fetch_i(tc, cont, res);
+        cs->fetch_u(tc, cont, res);
     else
         res->u64 = MVM_repr_get_uint(tc, cont);
 }
@@ -845,6 +930,15 @@ void MVM_6model_container_assign_i(MVMThreadContext *tc, MVMObject *cont, MVMint
         MVM_exception_throw_adhoc(tc, "Cannot assign to an immutable value");
 }
 
+/* Checks we have a container, and provided we do, assigns an uint into it. */
+void MVM_6model_container_assign_u(MVMThreadContext *tc, MVMObject *cont, MVMuint64 value) {
+    const MVMContainerSpec *cs = STABLE(cont)->container_spec;
+    if (cs && IS_CONCRETE(cont))
+        cs->store_u(tc, cont, value);
+    else
+        MVM_exception_throw_adhoc(tc, "Cannot assign to an immutable value");
+}
+
 /* Checks we have a container, and provided we do, assigns a num into it. */
 void MVM_6model_container_assign_n(MVMThreadContext *tc, MVMObject *cont, MVMnum64 value) {
     const MVMContainerSpec *cs = STABLE(cont)->container_spec;
@@ -897,8 +991,8 @@ MVMObject * MVM_6model_container_atomic_load(MVMThreadContext *tc, MVMObject *co
     if (IS_CONCRETE(cont)) {
         MVMContainerSpec const *cs = cont->st->container_spec;
         if (cs) {
-            if (cs->atomic_load)
-                return cs->atomic_load(tc, cont);
+            if (cs->load_atomic)
+                return cs->load_atomic(tc, cont);
             else
                 MVM_exception_throw_adhoc(tc,
                     "A %s container does not know how to do an atomic load",
@@ -921,8 +1015,8 @@ void MVM_6model_container_atomic_store(MVMThreadContext *tc, MVMObject *cont, MV
     if (IS_CONCRETE(cont)) {
         MVMContainerSpec const *cs = cont->st->container_spec;
         if (cs) {
-            if (cs->atomic_store)
-                cs->atomic_store(tc, cont, value);
+            if (cs->store_atomic)
+                cs->store_atomic(tc, cont, value);
             else
                 MVM_exception_throw_adhoc(tc,
                     "A %s container does not know how to do an atomic store",
@@ -965,7 +1059,7 @@ static AO_t * native_ref_as_atomic_i(MVMThreadContext *tc, MVMObject *cont) {
 
 MVMint64 MVM_6model_container_cas_i(MVMThreadContext *tc, MVMObject *cont,
                                     MVMint64 expected, MVMint64 value) {
-    return (MVMint64)MVM_cas(native_ref_as_atomic_i(tc, cont), (AO_t)expected, (AO_t)value);
+    return (MVMint64)MVM_cas(native_ref_as_atomic_i(tc, cont), AO_CAST(expected), AO_CAST(value));
 }
 
 MVMint64 MVM_6model_container_atomic_load_i(MVMThreadContext *tc, MVMObject *cont) {
@@ -985,5 +1079,5 @@ MVMint64 MVM_6model_container_atomic_dec(MVMThreadContext *tc, MVMObject *cont)
 }
 
 MVMint64 MVM_6model_container_atomic_add(MVMThreadContext *tc, MVMObject *cont, MVMint64 value) {
-    return (MVMint64)MVM_add(native_ref_as_atomic_i(tc, cont), (AO_t)value);
+    return (MVMint64)MVM_add(native_ref_as_atomic_i(tc, cont), AO_CAST(value));
 }
