@@ -105,6 +105,19 @@ MVMint64 MVM_repr_at_pos_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx) {
     return value.i64;
 }
 
+MVMuint64 MVM_repr_at_pos_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx) {
+    MVMRegister value;
+    if (REPR(obj)->ID == MVM_REPR_ID_VMArray) {
+        MVM_VMArray_at_pos(tc, STABLE(obj), obj, OBJECT_BODY(obj),
+            idx, &value, MVM_reg_uint64);
+    }
+    else {
+        REPR(obj)->pos_funcs.at_pos(tc, STABLE(obj), obj, OBJECT_BODY(obj),
+            idx, &value, MVM_reg_uint64);
+    }
+    return value.u64;
+}
+
 MVMnum64 MVM_repr_at_pos_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx) {
     MVMRegister value;
     if (REPR(obj)->ID == MVM_REPR_ID_VMArray) {
@@ -164,6 +177,12 @@ MVMint64 MVM_repr_at_pos_multidim_i(MVMThreadContext *tc, MVMObject *obj, MVMObj
     return r.i64;
 }
 
+MVMuint64 MVM_repr_at_pos_multidim_u(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices) {
+    MVMRegister r;
+    at_pos_multidim(tc, obj, indices, &r, MVM_reg_uint64);
+    return r.u64;
+}
+
 MVMnum64 MVM_repr_at_pos_multidim_n(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices) {
     MVMRegister r;
     at_pos_multidim(tc, obj, indices, &r, MVM_reg_num64);
@@ -194,6 +213,12 @@ MVMint64 MVM_repr_at_pos_2d_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx
     return r.i64;
 }
 
+MVMuint64 MVM_repr_at_pos_2d_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2) {
+    MVMRegister r;
+    at_pos_2d(tc, obj, idx1, idx2, &r, MVM_reg_uint64);
+    return r.u64;
+}
+
 MVMnum64 MVM_repr_at_pos_2d_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2) {
     MVMRegister r;
     at_pos_2d(tc, obj, idx1, idx2, &r, MVM_reg_num64);
@@ -224,6 +249,12 @@ MVMint64 MVM_repr_at_pos_3d_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx
     return r.i64;
 }
 
+MVMuint64 MVM_repr_at_pos_3d_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMint64 idx3) {
+    MVMRegister r;
+    at_pos_3d(tc, obj, idx1, idx2, idx3, &r, MVM_reg_uint64);
+    return r.u64;
+}
+
 MVMnum64 MVM_repr_at_pos_3d_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMint64 idx3) {
     MVMRegister r;
     at_pos_3d(tc, obj, idx1, idx2, idx3, &r, MVM_reg_num64);
@@ -249,6 +280,13 @@ void MVM_repr_bind_pos_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx, MVM
         idx, val, MVM_reg_int64);
 }
 
+void MVM_repr_bind_pos_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx, MVMuint64 value) {
+    MVMRegister val;
+    val.u64 = value;
+    REPR(obj)->pos_funcs.bind_pos(tc, STABLE(obj), obj, OBJECT_BODY(obj),
+        idx, val, MVM_reg_uint64);
+}
+
 void MVM_repr_bind_pos_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx, MVMnum64 value) {
     MVMRegister val;
     val.n64 = value;
@@ -290,6 +328,11 @@ void MVM_repr_bind_pos_multidim_i(MVMThreadContext *tc, MVMObject *obj, MVMObjec
     r.i64 = value;
     bind_pos_multidim(tc, obj, indices, r, MVM_reg_int64);
 }
+void MVM_repr_bind_pos_multidim_u(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices, MVMuint64 value) {
+    MVMRegister r;
+    r.u64 = value;
+    bind_pos_multidim(tc, obj, indices, r, MVM_reg_uint64);
+}
 void MVM_repr_bind_pos_multidim_n(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices, MVMnum64 value) {
     MVMRegister r;
     r.n64 = value;
@@ -317,6 +360,11 @@ void MVM_repr_bind_pos_2d_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1,
     r.i64 = value;
     bind_pos_2d(tc, obj, idx1, idx2, r, MVM_reg_int64);
 }
+void MVM_repr_bind_pos_2d_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMuint64 value) {
+    MVMRegister r;
+    r.u64 = value;
+    bind_pos_2d(tc, obj, idx1, idx2, r, MVM_reg_uint64);
+}
 void MVM_repr_bind_pos_2d_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMnum64 value) {
     MVMRegister r;
     r.n64 = value;
@@ -344,6 +392,11 @@ void MVM_repr_bind_pos_3d_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1,
     r.i64 = value;
     bind_pos_3d(tc, obj, idx1, idx2, idx3, r, MVM_reg_int64);
 }
+void MVM_repr_bind_pos_3d_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMint64 idx3, MVMuint64 value) {
+    MVMRegister r;
+    r.u64 = value;
+    bind_pos_3d(tc, obj, idx1, idx2, idx3, r, MVM_reg_uint64);
+}
 void MVM_repr_bind_pos_3d_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx1, MVMint64 idx2, MVMint64 idx3, MVMnum64 value) {
     MVMRegister r;
     r.n64 = value;
@@ -694,6 +747,18 @@ MVM_PUBLIC MVMint64 MVM_repr_get_attr_i(MVMThreadContext *tc, MVMObject *object,
     return result_reg.i64;
 }
 
+MVM_PUBLIC MVMuint64 MVM_repr_get_attr_u(MVMThreadContext *tc, MVMObject *object, MVMObject *type,
+                                           MVMString *name, MVMint16 hint) {
+    MVMRegister result_reg;
+    if (!IS_CONCRETE(object))
+        MVM_exception_throw_adhoc(tc, "Cannot look up attributes in a %s type object. Did you forget a '.new'?", MVM_6model_get_debug_name(tc, object));
+    REPR(object)->attr_funcs.get_attribute(tc,
+            STABLE(object), object, OBJECT_BODY(object),
+            type, name,
+            hint, &result_reg, MVM_reg_uint64);
+    return result_reg.u64;
+}
+
 MVM_PUBLIC MVMnum64 MVM_repr_get_attr_n(MVMThreadContext *tc, MVMObject *object, MVMObject *type,
                                            MVMString *name, MVMint16 hint) {
     MVMRegister result_reg;
