@@ -3,6 +3,8 @@
 /* This representation's function pointer table. */
 static const MVMREPROps CPPStruct_this_repr;
 
+static void free_repr_data(MVMCPPStructREPRData *repr_data);
+
 /* Locates all of the attributes. Puts them onto a flattened, ordered
  * list of attributes (populating the passed flat_list). Also builds
  * the index mapping for doing named lookups. Note index is not related
@@ -153,6 +155,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
 
             if (num_dimensions > 1) {
                 MVM_gc_allocate_gen2_default_clear(tc);
+                free_repr_data(repr_data);
                 MVM_exception_throw_adhoc(tc,
                     "Only one dimensions supported in CPPStruct attribute");
             }
@@ -163,6 +166,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                 MVMint32  type_id    = REPR(type)->ID;
                 if (spec->inlineable == MVM_STORAGE_SPEC_INLINED &&
                         (spec->boxed_primitive == MVM_STORAGE_SPEC_BP_INT ||
+                         spec->boxed_primitive == MVM_STORAGE_SPEC_BP_UINT64 ||
                          spec->boxed_primitive == MVM_STORAGE_SPEC_BP_NUM)) {
                     /* It's a boxed int or num; pretty easy. It'll just live in the
                      * body of the struct. Instead of masking in i here (which
@@ -208,6 +212,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                         MVMCArrayREPRData *carray_repr_data = (MVMCArrayREPRData *)STABLE(type)->REPR_data;
                         if (!carray_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CPPStruct: can't inline a CArray attribute before its type's definition");
                         }
@@ -240,6 +245,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                         MVMCStructREPRData *cstruct_repr_data = (MVMCStructREPRData *)STABLE(type)->REPR_data;
                         if (!cstruct_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CPPStruct: can't inline a CStruct attribute before its type's definition");
                         }
@@ -257,6 +263,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                         MVMCPPStructREPRData *cppstruct_repr_data = (MVMCPPStructREPRData *)STABLE(type)->REPR_data;
                         if (!cppstruct_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CPPStruct: can't inline a CPPStruct attribute before its type's definition");
                         }
@@ -274,6 +281,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                         MVMCUnionREPRData *cunion_repr_data = (MVMCUnionREPRData *)STABLE(type)->REPR_data;
                         if (!cunion_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CPPStruct: can't inline a CUnion attribute before its type's definition");
                         }
@@ -290,6 +298,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
                 }
                 else {
                     MVM_gc_allocate_gen2_default_clear(tc);
+                    free_repr_data(repr_data);
                     MVM_exception_throw_adhoc(tc,
                         "CPPStruct representation only handles attributes of type:\n"
                         "  (u)int8, (u)int16, (u)int32, (u)int64, (u)long, (u)longlong, num32, num64, (s)size_t, bool, Str\n"
@@ -298,13 +307,15 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMSTable *st,
             }
             else {
                 MVM_gc_allocate_gen2_default_clear(tc);
+                free_repr_data(repr_data);
                 MVM_exception_throw_adhoc(tc,
                     "CPPStruct representation requires the types of all attributes to be specified");
             }
 
             if (bits % 8) {
                 MVM_gc_allocate_gen2_default_clear(tc);
-                 MVM_exception_throw_adhoc(tc,
+                free_repr_data(repr_data);
+                MVM_exception_throw_adhoc(tc,
                     "CPPStruct only supports native types that are a multiple of 8 bits wide (was passed: %"PRId32")", bits);
             }
 
@@ -379,8 +390,8 @@ static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
 /* Composes the representation. */
 static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *repr_info) {
     /* Compute allocation strategy. */
-    MVMCPPStructREPRData *repr_data = MVM_calloc(1, sizeof(MVMCPPStructREPRData));
     MVMObject *attr_info = MVM_repr_at_key_o(tc, repr_info, tc->instance->str_consts.attribute);
+    MVMCPPStructREPRData *repr_data = MVM_calloc(1, sizeof(MVMCPPStructREPRData));
     MVM_gc_allocate_gen2_default_set(tc);
     compute_allocation_strategy(tc, st, attr_info, repr_data);
     MVM_gc_allocate_gen2_default_clear(tc);
@@ -393,7 +404,7 @@ static void initialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, voi
 
     /* Allocate object body. */
     MVMCPPStructBody *body = (MVMCPPStructBody *)data;
-    body->cppstruct = MVM_calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
+    body->cppstruct = calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
 
     /* Allocate child obj array. */
     if (repr_data->num_child_objs > 0)
@@ -405,7 +416,7 @@ static void initialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, voi
         MVMint32 i;
         for (i = 0; repr_data->initialize_slots[i] >= 0; i++) {
             MVMint32  offset = repr_data->struct_offsets[repr_data->initialize_slots[i]];
-            MVMSTable *st     = repr_data->flattened_stables[repr_data->initialize_slots[i]];
+            MVMSTable *st    = repr_data->flattened_stables[repr_data->initialize_slots[i]];
             st->REPR->initialize(tc, st, root, (char *)body->cppstruct + offset);
         }
     }
@@ -515,6 +526,14 @@ static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CPPStruct: invalid native get of object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                result_reg->u64 = attr_st->REPR->box_funcs.get_uint(tc, attr_st, root,
+                    ((char *)body->cppstruct) + repr_data->struct_offsets[slot]);
+            else
+                MVM_exception_throw_adhoc(tc, "CPPStruct: invalid native get of object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 result_reg->n64 = attr_st->REPR->box_funcs.get_num(tc, attr_st, root,
@@ -626,6 +645,14 @@ static void bind_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CPPStruct: invalid native binding to object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                attr_st->REPR->box_funcs.set_uint(tc, attr_st, root,
+                    ((char *)body->cppstruct) + repr_data->struct_offsets[slot], value_reg.u64);
+            else
+                MVM_exception_throw_adhoc(tc, "CPPStruct: invalid native binding to object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 attr_st->REPR->box_funcs.set_num(tc, attr_st, root,
@@ -698,10 +725,7 @@ static void gc_mark_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMGCWorklist
     }
 }
 
-/* Free representation data. */
-static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
-    MVMCPPStructREPRData *repr_data = (MVMCPPStructREPRData *)st->REPR_data;
-
+static void free_repr_data(MVMCPPStructREPRData *repr_data) {
     /* May not have survived to composition. */
     if (repr_data == NULL)
         return;
@@ -715,7 +739,13 @@ static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
         MVM_free(repr_data->initialize_slots);
     }
 
-    MVM_free(st->REPR_data);
+    MVM_free(repr_data);
+}
+
+/* Free representation data. */
+static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
+    MVMCPPStructREPRData *repr_data = (MVMCPPStructREPRData *)st->REPR_data;
+    free_repr_data(repr_data);
 }
 
 /* This is called to do any cleanup of resources when an object gets
@@ -727,7 +757,7 @@ static void gc_cleanup(MVMThreadContext *tc, MVMSTable *st, void *data) {
     /* XXX For some reason, this causes crashes at the moment. Need to
      * work out why. */
     /*if (body->cppstruct)
-        MVM_free(body->cppstruct);*/
+        free(body->cppstruct);*/
 }
 
 /* Called by the VM in order to free memory associated with this object. */
@@ -795,9 +825,7 @@ static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerial
     MVMint32 i, num_classes, num_slots;
 
     repr_data->struct_size    = MVM_serialization_read_int(tc, reader);
-    if (reader->root.version >= 17) {
-        repr_data->struct_align = MVM_serialization_read_int(tc, reader);
-    }
+    repr_data->struct_align   = MVM_serialization_read_int(tc, reader);
     repr_data->num_attributes = MVM_serialization_read_int(tc, reader);
     repr_data->num_child_objs = MVM_serialization_read_int(tc, reader);
 
