@@ -275,6 +275,7 @@ static void snapshot_frame_callees(MVMThreadContext *tc, MVMFrame *f) {
         MVMSpeshCandidate *cand = f->caller->spesh_cand;
         MVMFrameExtra *extra = MVM_frame_extra(tc, f);
         extra->caller_info_needed = 1;
+        extra->caller_pos_needed = 1;
         if (cand && cand->body.num_inlines) {
             if (cand->body.jitcode) {
                 if (extra->caller_jit_position)
@@ -285,7 +286,7 @@ static void snapshot_frame_callees(MVMThreadContext *tc, MVMFrame *f) {
                 if (extra->caller_deopt_idx)
                     return;
                 /* Store with + 1 to avoid semi-predicate problem. */
-                extra->caller_deopt_idx = MVM_spesh_deopt_find_inactive_frame_deopt_idx(tc, f->caller) + 1;
+                extra->caller_deopt_idx = 1 + MVM_spesh_deopt_find_inactive_frame_deopt_idx(tc, f->caller, cand);
             }
         }
         f = f->caller;
@@ -297,6 +298,19 @@ MVMObject * MVM_context_from_frame(MVMThreadContext *tc, MVMFrame *f) {
     MVMObject *ctx;
     f = MVM_frame_force_to_heap(tc, f);
     snapshot_frame_callees(tc, f);
+    MVMROOT(tc, f, {
+        ctx = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTContext);
+        MVM_ASSIGN_REF(tc, &(ctx->header), ((MVMContext *)ctx)->body.context, f);
+        ((MVMContext *)ctx)->body.traversable = 1;
+    });
+    return ctx;
+}
+
+/* Creates a MVMContent wrapper object around an MVMFrame that only allows
+ * access to the current frame and does not support traversal. */
+MVMObject * MVM_context_from_frame_non_traversable(MVMThreadContext *tc, MVMFrame *f) {
+    MVMObject *ctx;
+    f = MVM_frame_force_to_heap(tc, f);
     MVMROOT(tc, f, {
         ctx = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTContext);
         MVM_ASSIGN_REF(tc, &(ctx->header), ((MVMContext *)ctx)->body.context, f);
@@ -318,6 +332,11 @@ static MVMint32 traversal_exists(MVMThreadContext *tc, MVMFrame *base, MVMuint8
 /* Try to get a context with the specified traversal applied. Ensures that the
  * traversal would lead to a result, and returns a NULL context if not. */
 MVMObject * MVM_context_apply_traversal(MVMThreadContext *tc, MVMContext *ctx, MVMuint8 traversal) {
+    /* Ensure we've got a traversable context. */
+    if (!ctx->body.traversable)
+        MVM_exception_throw_adhoc(tc,
+                "Cannot move to outers or callers with non-traversable context");
+
     /* Create new traversals array with the latest one at the end. */
     MVMuint32 new_num_traversals = ctx->body.num_traversals + 1;
     MVMuint8 *new_traversals = MVM_malloc(new_num_traversals);
@@ -335,6 +354,7 @@ MVMObject * MVM_context_apply_traversal(MVMThreadContext *tc, MVMContext *ctx, M
         MVM_ASSIGN_REF(tc, &(result->common.header), result->body.context, ctx->body.context);
         result->body.traversals = new_traversals;
         result->body.num_traversals = new_num_traversals;
+        result->body.traversable = 1;
         return (MVMObject *)result;
     }
     else {
@@ -419,7 +439,7 @@ MVMObject * MVM_context_lexical_lookup(MVMThreadContext *tc, MVMContext *ctx, MV
     MVMSpeshFrameWalker fw;
     MVM_spesh_frame_walker_init_for_outers(tc, &fw, ctx->body.context);
     if (apply_traversals(tc, &fw, ctx->body.traversals, ctx->body.num_traversals)) {
-        MVMRegister *result = MVM_frame_lexical_lookup_using_frame_walker(tc, &fw, name);
+        MVMRegister *result = MVM_frame_lexical_lookup_using_frame_walker(tc, &fw, name, MVM_reg_obj);
         return result ? result->o : tc->instance->VMNull;
     }
     MVM_spesh_frame_walker_cleanup(tc, &fw);
@@ -443,7 +463,7 @@ MVMObject * MVM_context_caller_lookup(MVMThreadContext *tc, MVMContext *ctx, MVM
     MVMSpeshFrameWalker fw;
     MVM_spesh_frame_walker_init(tc, &fw, ctx->body.context, 1);
     if (apply_traversals(tc, &fw, ctx->body.traversals, ctx->body.num_traversals)) {
-        MVMRegister *result = MVM_frame_lexical_lookup_using_frame_walker(tc, &fw, name);
+        MVMRegister *result = MVM_frame_lexical_lookup_using_frame_walker(tc, &fw, name, MVM_reg_obj);
         return result ? result->o : tc->instance->VMNull;
     }
     MVM_spesh_frame_walker_cleanup(tc, &fw);
