@@ -108,9 +108,11 @@ static void shift(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *da
             ; /* Sigh, C99 won't let me put a declaration here. */
             MVMStrHashTable *hashtable = &(((MVMHash *)target)->body.hashtable);
 #if HASH_DEBUG_ITER
-            if (body->hash_state.curr.owner != hashtable->ht_id) {
+            struct MVMStrHashTableControl *control = hashtable->table;
+            MVMuint64 ht_id = control ? control->ht_id : 0;
+            if (body->hash_state.curr.owner != ht_id) {
                 MVM_oops(tc, "MVMIter shift called with an iterator from a different hash table: %016" PRIx64 " != %016" PRIx64,
-                         body->hash_state.curr.owner, hashtable->ht_id);
+                         body->hash_state.curr.owner, ht_id);
             }
             /* OK, to implement "delete at current iterator position" we need
              * to cheat somewhat. */
@@ -120,7 +122,7 @@ static void shift(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *da
                  * valid (but has already been advanced beyond pos, so we
                  * can't perform this test on it). So "fix up" its state to pass
                  * muster with the HASH_DEBUG_ITER sanity tests. */
-                body->hash_state.next.serial = hashtable->serial;
+                body->hash_state.next.serial = control->serial;
             }
 #endif
             body->hash_state.curr = body->hash_state.next;
@@ -279,10 +281,12 @@ MVMString * MVM_iterkey_s(MVMThreadContext *tc, MVMIter *iterator) {
     MVMStrHashTable *hashtable = &(((MVMHash *)iterator->body.target)->body.hashtable);
 
 #if HASH_DEBUG_ITER
-        if (iterator->body.hash_state.next.owner != hashtable->ht_id) {
-            MVM_oops(tc, "MVM_itereky_s called with an iterator from a different hash table: %016" PRIx64 " != %016" PRIx64,
-                     iterator->body.hash_state.next.owner, hashtable->ht_id);
-        }
+    struct MVMStrHashTableControl *control = hashtable->table;
+    MVMuint64 ht_id = control ? control->ht_id : 0;
+    if (iterator->body.hash_state.next.owner != ht_id) {
+        MVM_oops(tc, "MVM_itereky_s called with an iterator from a different hash table: %016" PRIx64 " != %016" PRIx64,
+                 iterator->body.hash_state.next.owner, ht_id);
+    }
 #endif
 
     if (MVM_str_hash_at_end(tc, hashtable, iterator->body.hash_state.curr)
@@ -310,9 +314,11 @@ MVMObject * MVM_iterval(MVMThreadContext *tc, MVMIter *iterator) {
         MVMStrHashTable *hashtable = &(((MVMHash *)iterator->body.target)->body.hashtable);
 
 #if HASH_DEBUG_ITER
-        if (iterator->body.hash_state.next.owner != hashtable->ht_id) {
+        struct MVMStrHashTableControl *control = hashtable->table;
+        MVMuint64 ht_id = control ? control->ht_id : 0;
+        if (iterator->body.hash_state.next.owner != ht_id) {
         MVM_oops(tc, "MVM_iterval called with an iterator from a different hash table: %016" PRIx64 " != %016" PRIx64,
-                 iterator->body.hash_state.next.owner, hashtable->ht_id);
+                 iterator->body.hash_state.next.owner, ht_id);
         }
 #endif
 
