@@ -33,10 +33,12 @@ static void serialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerializ
     if (repr_data) {
         MVM_serialization_write_int(tc, writer, repr_data->primitive_type);
         MVM_serialization_write_int(tc, writer, repr_data->ref_kind);
+        MVM_serialization_write_int(tc, writer, repr_data->is_unsigned);
     }
     else {
         MVM_serialization_write_int(tc, writer, 0);
         MVM_serialization_write_int(tc, writer, 0);
+        MVM_serialization_write_int(tc, writer, 0);
     }
 }
 
@@ -45,6 +47,7 @@ static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerial
     MVMNativeRefREPRData *repr_data = MVM_malloc(sizeof(MVMNativeRefREPRData));
     repr_data->primitive_type = MVM_serialization_read_int(tc, reader);
     repr_data->ref_kind       = MVM_serialization_read_int(tc, reader);
+    repr_data->is_unsigned    = MVM_serialization_read_int(tc, reader);
     st->REPR_data = repr_data;
 }
 
@@ -95,7 +98,8 @@ static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *info_hash) {
     MVMObject *info = MVM_repr_at_key_o(tc, info_hash, str_consts->nativeref);
     if (IS_CONCRETE(info)) {
         MVMObject *type    = MVM_repr_at_key_o(tc, info, str_consts->type);
-        MVMuint16  prim    = REPR(type)->get_storage_spec(tc, STABLE(type))->boxed_primitive;
+        const MVMStorageSpec *spec = REPR(type)->get_storage_spec(tc, STABLE(type));
+        MVMuint16  prim    = spec->boxed_primitive;
         if (prim != MVM_STORAGE_SPEC_BP_NONE) {
             MVMObject *refkind = MVM_repr_at_key_o(tc, info, str_consts->refkind);
             if (IS_CONCRETE(refkind)) {
@@ -121,6 +125,7 @@ static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *info_hash) {
                 repr_data = MVM_malloc(sizeof(MVMNativeRefREPRData));
                 repr_data->primitive_type = prim;
                 repr_data->ref_kind       = kind;
+                repr_data->is_unsigned    = spec->is_unsigned;
                 st->REPR_data             = repr_data;
             }
             else {
@@ -187,7 +192,7 @@ void MVM_nativeref_ensure(MVMThreadContext *tc, MVMObject *type, MVMuint16 wantp
         if (!repr_data)
             MVM_exception_throw_adhoc(tc, "%s set to NativeRef that is not yet composed", guilty);
         if (repr_data->primitive_type != wantprim)
-            MVM_exception_throw_adhoc(tc, "%s set to NativeRef of wrong primitive type", guilty);
+            MVM_exception_throw_adhoc(tc, "%s set to NativeRef of wrong primitive type, got %d, expected %d", guilty, repr_data->primitive_type, wantprim);
         if (repr_data->ref_kind != wantkind)
             MVM_exception_throw_adhoc(tc, "%s set to NativeRef of wrong reference kind", guilty);
     }
@@ -226,8 +231,8 @@ MVMObject * MVM_nativeref_lex_i(MVMThreadContext *tc, MVMuint16 outers, MVMuint1
     ref_type = MVM_hll_current(tc)->int_lex_ref;
     if (ref_type) {
         MVMFrame  *f = get_lexical_outer(tc, outers);
-        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->lexical_types
-            ? f->spesh_cand->lexical_types
+        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->body.lexical_types
+            ? f->spesh_cand->body.lexical_types
             : f->static_info->body.lexical_types;
         MVMuint16 type = lexical_types[idx];
         if (type != MVM_reg_int64 && type != MVM_reg_int32 &&
@@ -239,14 +244,31 @@ MVMObject * MVM_nativeref_lex_i(MVMThreadContext *tc, MVMuint16 outers, MVMuint1
     }
     MVM_exception_throw_adhoc(tc, "No int lexical reference type registered for current HLL");
 }
+MVMObject * MVM_nativeref_lex_u(MVMThreadContext *tc, MVMuint16 outers, MVMuint16 idx) {
+    MVMObject *ref_type;
+    MVM_frame_force_to_heap(tc, tc->cur_frame);
+    ref_type = MVM_hll_current(tc)->uint_lex_ref;
+    if (ref_type) {
+        MVMFrame  *f = get_lexical_outer(tc, outers);
+        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->body.lexical_types
+            ? f->spesh_cand->body.lexical_types
+            : f->static_info->body.lexical_types;
+        MVMuint16 type = lexical_types[idx];
+        if (type != MVM_reg_uint64 && type != MVM_reg_uint32 &&
+                type != MVM_reg_uint16 && type != MVM_reg_uint8)
+            MVM_exception_throw_adhoc(tc, "getlexref_u: lexical is not an uint");
+        return lex_ref(tc, ref_type, f, idx, type);
+    }
+    MVM_exception_throw_adhoc(tc, "No uint lexical reference type registered for current HLL");
+}
 MVMObject * MVM_nativeref_lex_n(MVMThreadContext *tc, MVMuint16 outers, MVMuint16 idx) {
     MVMObject *ref_type;
     MVM_frame_force_to_heap(tc, tc->cur_frame);
     ref_type = MVM_hll_current(tc)->num_lex_ref;
     if (ref_type) {
         MVMFrame  *f = get_lexical_outer(tc, outers);
-        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->lexical_types
-            ? f->spesh_cand->lexical_types
+        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->body.lexical_types
+            ? f->spesh_cand->body.lexical_types
             : f->static_info->body.lexical_types;
         MVMuint16 type = lexical_types[idx];
         if (type != MVM_reg_num64 && type != MVM_reg_num32)
@@ -261,8 +283,8 @@ MVMObject * MVM_nativeref_lex_s(MVMThreadContext *tc, MVMuint16 outers, MVMuint1
     ref_type = MVM_hll_current(tc)->str_lex_ref;
     if (ref_type) {
         MVMFrame  *f = get_lexical_outer(tc, outers);
-        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->lexical_types
-            ? f->spesh_cand->lexical_types
+        MVMuint16 *lexical_types = f->spesh_cand && f->spesh_cand->body.lexical_types
+            ? f->spesh_cand->body.lexical_types
             : f->static_info->body.lexical_types;
         if (lexical_types[idx] != MVM_reg_str)
             MVM_exception_throw_adhoc(tc, "getlexref_s: lexical is not a str (%d, %d)", outers, idx);
@@ -361,6 +383,12 @@ MVMObject * MVM_nativeref_attr_i(MVMThreadContext *tc, MVMObject *obj, MVMObject
         return attrref(tc, ref_type, obj, class_handle, name);
     MVM_exception_throw_adhoc(tc, "No int attribute reference type registered for current HLL");
 }
+MVMObject * MVM_nativeref_attr_u(MVMThreadContext *tc, MVMObject *obj, MVMObject *class_handle, MVMString *name) {
+    MVMObject *ref_type = MVM_hll_current(tc)->uint_attr_ref;
+    if (ref_type)
+        return attrref(tc, ref_type, obj, class_handle, name);
+    MVM_exception_throw_adhoc(tc, "No uint attribute reference type registered for current HLL");
+}
 MVMObject * MVM_nativeref_attr_n(MVMThreadContext *tc, MVMObject *obj, MVMObject *class_handle, MVMString *name) {
     MVMObject *ref_type = MVM_hll_current(tc)->num_attr_ref;
     if (ref_type)
@@ -390,6 +418,12 @@ MVMObject * MVM_nativeref_pos_i(MVMThreadContext *tc, MVMObject *obj, MVMint64 i
         return posref(tc, ref_type, obj, idx);
     MVM_exception_throw_adhoc(tc, "No int positional reference type registered for current HLL");
 }
+MVMObject * MVM_nativeref_pos_u(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx) {
+    MVMObject *ref_type = MVM_hll_current(tc)->uint_pos_ref;
+    if (ref_type)
+        return posref(tc, ref_type, obj, idx);
+    MVM_exception_throw_adhoc(tc, "No uint positional reference type registered for current HLL");
+}
 MVMObject * MVM_nativeref_pos_n(MVMThreadContext *tc, MVMObject *obj, MVMint64 idx) {
     MVMObject *ref_type = MVM_hll_current(tc)->num_pos_ref;
     if (ref_type)
@@ -419,6 +453,12 @@ MVMObject * MVM_nativeref_multidim_i(MVMThreadContext *tc, MVMObject *obj, MVMOb
         return md_posref(tc, ref_type, obj, indices);
     MVM_exception_throw_adhoc(tc, "No int multidim positional reference type registered for current HLL");
 }
+MVMObject * MVM_nativeref_multidim_u(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices) {
+    MVMObject *ref_type = MVM_hll_current(tc)->uint_multidim_ref;
+    if (ref_type)
+        return md_posref(tc, ref_type, obj, indices);
+    MVM_exception_throw_adhoc(tc, "No int multidim positional reference type registered for current HLL");
+}
 MVMObject * MVM_nativeref_multidim_n(MVMThreadContext *tc, MVMObject *obj, MVMObject *indices) {
     MVMObject *ref_type = MVM_hll_current(tc)->num_multidim_ref;
     if (ref_type)
@@ -478,6 +518,11 @@ MVMint64 MVM_nativeref_read_attribute_i(MVMThreadContext *tc, MVMObject *ref_obj
     return MVM_repr_get_attr_i(tc, ref->body.u.attribute.obj,
         ref->body.u.attribute.class_handle, ref->body.u.attribute.name, MVM_NO_HINT);
 }
+MVMuint64 MVM_nativeref_read_attribute_u(MVMThreadContext *tc, MVMObject *ref_obj) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    return MVM_repr_get_attr_u(tc, ref->body.u.attribute.obj,
+        ref->body.u.attribute.class_handle, ref->body.u.attribute.name, MVM_NO_HINT);
+}
 MVMnum64 MVM_nativeref_read_attribute_n(MVMThreadContext *tc, MVMObject *ref_obj) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     return MVM_repr_get_attr_n(tc, ref->body.u.attribute.obj,
@@ -492,6 +537,10 @@ MVMint64 MVM_nativeref_read_positional_i(MVMThreadContext *tc, MVMObject *ref_ob
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     return MVM_repr_at_pos_i(tc, ref->body.u.positional.obj, ref->body.u.positional.idx);
 }
+MVMuint64 MVM_nativeref_read_positional_u(MVMThreadContext *tc, MVMObject *ref_obj) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    return MVM_repr_at_pos_u(tc, ref->body.u.positional.obj, ref->body.u.positional.idx);
+}
 MVMnum64 MVM_nativeref_read_positional_n(MVMThreadContext *tc, MVMObject *ref_obj) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     return MVM_repr_at_pos_n(tc, ref->body.u.positional.obj, ref->body.u.positional.idx);
@@ -504,6 +553,10 @@ MVMint64 MVM_nativeref_read_multidim_i(MVMThreadContext *tc, MVMObject *ref_obj)
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     return MVM_repr_at_pos_multidim_i(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices);
 }
+MVMuint64 MVM_nativeref_read_multidim_u(MVMThreadContext *tc, MVMObject *ref_obj) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    return MVM_repr_at_pos_multidim_u(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices);
+}
 MVMnum64 MVM_nativeref_read_multidim_n(MVMThreadContext *tc, MVMObject *ref_obj) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     return MVM_repr_at_pos_multidim_n(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices);
@@ -568,6 +621,13 @@ void MVM_nativeref_write_attribute_i(MVMThreadContext *tc, MVMObject *ref_obj, M
     MVM_repr_bind_attr_inso(tc, ref->body.u.attribute.obj, ref->body.u.attribute.class_handle,
         ref->body.u.attribute.name, MVM_NO_HINT, r, MVM_reg_int64);
 }
+void MVM_nativeref_write_attribute_u(MVMThreadContext *tc, MVMObject *ref_obj, MVMint64 value) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    MVMRegister r;
+    r.i64 = value;
+    MVM_repr_bind_attr_inso(tc, ref->body.u.attribute.obj, ref->body.u.attribute.class_handle,
+        ref->body.u.attribute.name, MVM_NO_HINT, r, MVM_reg_uint64);
+}
 void MVM_nativeref_write_attribute_n(MVMThreadContext *tc, MVMObject *ref_obj, MVMnum64 value) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     MVMRegister r;
@@ -586,6 +646,10 @@ void MVM_nativeref_write_positional_i(MVMThreadContext *tc, MVMObject *ref_obj,
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     MVM_repr_bind_pos_i(tc, ref->body.u.positional.obj, ref->body.u.positional.idx, value);
 }
+void MVM_nativeref_write_positional_u(MVMThreadContext *tc, MVMObject *ref_obj, MVMuint64 value) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    MVM_repr_bind_pos_u(tc, ref->body.u.positional.obj, ref->body.u.positional.idx, value);
+}
 void MVM_nativeref_write_positional_n(MVMThreadContext *tc, MVMObject *ref_obj, MVMnum64 value) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     MVM_repr_bind_pos_n(tc, ref->body.u.positional.obj, ref->body.u.positional.idx, value);
@@ -599,6 +663,10 @@ void MVM_nativeref_write_multidim_i(MVMThreadContext *tc, MVMObject *ref_obj, MV
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     MVM_repr_bind_pos_multidim_i(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices, value);
 }
+void MVM_nativeref_write_multidim_u(MVMThreadContext *tc, MVMObject *ref_obj, MVMuint64 value) {
+    MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
+    MVM_repr_bind_pos_multidim_u(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices, value);
+}
 void MVM_nativeref_write_multidim_n(MVMThreadContext *tc, MVMObject *ref_obj, MVMnum64 value) {
     MVMNativeRef *ref = (MVMNativeRef *)ref_obj;
     MVM_repr_bind_pos_multidim_n(tc, ref->body.u.multidim.obj, ref->body.u.multidim.indices, value);
