@@ -180,7 +180,7 @@ void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
             break;
         case MVM_ARRAY_I64:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos I64 expected int register");
             if (real_index >= body->elems)
                 value->i64 = 0;
             else
@@ -188,7 +188,7 @@ void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
             break;
         case MVM_ARRAY_I32:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos I32 expected int register");
             if (real_index >= body->elems)
                 value->i64 = 0;
             else
@@ -196,7 +196,7 @@ void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
             break;
         case MVM_ARRAY_I16:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos I16 expected int register");
             if (real_index >= body->elems)
                 value->i64 = 0;
             else
@@ -204,7 +204,7 @@ void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
             break;
         case MVM_ARRAY_I8:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos I8 expected int register");
             if (real_index >= body->elems)
                 value->i64 = 0;
             else
@@ -227,36 +227,36 @@ void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
                 value->n64 = (MVMnum64)body->slots.n32[body->start + real_index];
             break;
         case MVM_ARRAY_U64:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos U64 expected int register, got %d instead", kind);
             if (real_index >= body->elems)
-                value->i64 = 0;
+                value->u64 = 0;
             else
-                value->i64 = (MVMint64)body->slots.u64[body->start + real_index];
+                value->u64 = (MVMint64)body->slots.u64[body->start + real_index];
             break;
         case MVM_ARRAY_U32:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos U32 expected int register");
             if (real_index >= body->elems)
-                value->i64 = 0;
+                value->u64 = 0;
             else
-                value->i64 = (MVMint64)body->slots.u32[body->start + real_index];
+                value->u64 = (MVMint64)body->slots.u32[body->start + real_index];
             break;
         case MVM_ARRAY_U16:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos U16 expected int register");
             if (real_index >= body->elems)
-                value->i64 = 0;
+                value->u64 = 0;
             else
-                value->i64 = (MVMint64)body->slots.u16[body->start + real_index];
+                value->u64 = (MVMint64)body->slots.u16[body->start + real_index];
             break;
         case MVM_ARRAY_U8:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: atpos U8 expected int register");
             if (real_index >= body->elems)
-                value->i64 = 0;
+                value->u64 = 0;
             else
-                value->i64 = (MVMint64)body->slots.u8[body->start + real_index];
+                value->u64 = (MVMint64)body->slots.u8[body->start + real_index];
             break;
         default:
             MVM_exception_throw_adhoc(tc, "MVMArray: Unhandled slot type, got '%s'", MVM_reg_get_debug_name(tc, repr_data->slot_type));
@@ -412,22 +412,22 @@ void MVM_VMArray_bind_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
             break;
         case MVM_ARRAY_I64:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos I64 expected int register");
             body->slots.i64[body->start + real_index] = value.i64;
             break;
         case MVM_ARRAY_I32:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos I32 expected int register");
             body->slots.i32[body->start + real_index] = (MVMint32)value.i64;
             break;
         case MVM_ARRAY_I16:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos I16 expected int register");
             body->slots.i16[body->start + real_index] = (MVMint16)value.i64;
             break;
         case MVM_ARRAY_I8:
             if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos I8 expected int register");
             body->slots.i8[body->start + real_index] = (MVMint8)value.i64;
             break;
         case MVM_ARRAY_N64:
@@ -441,23 +441,23 @@ void MVM_VMArray_bind_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
             body->slots.n32[body->start + real_index] = (MVMnum32)value.n64;
             break;
         case MVM_ARRAY_U64:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos U64 expected int register");
             body->slots.u64[body->start + real_index] = value.i64;
             break;
         case MVM_ARRAY_U32:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos U32 expected int register");
             body->slots.u32[body->start + real_index] = (MVMuint32)value.i64;
             break;
         case MVM_ARRAY_U16:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos U16 expected int register");
             body->slots.u16[body->start + real_index] = (MVMuint16)value.i64;
             break;
         case MVM_ARRAY_U8:
-            if (kind != MVM_reg_int64)
-                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos expected int register");
+            if (kind != MVM_reg_uint64 && kind != MVM_reg_int64)
+                MVM_exception_throw_adhoc(tc, "MVMArray: bindpos U8 expected int register");
             body->slots.u8[body->start + real_index] = (MVMuint8)value.i64;
             break;
         default:
@@ -807,6 +807,35 @@ static void shift(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *da
     exit_single_user(tc, body);
 }
 
+static MVMuint16 slot_type_to_kind(MVMuint8 slot_type) {
+    switch (slot_type) {
+        case MVM_ARRAY_OBJ:
+            return MVM_reg_obj;
+            break;
+        case MVM_ARRAY_STR:
+            return MVM_reg_str;
+            break;
+        case MVM_ARRAY_I64:
+        case MVM_ARRAY_I32:
+        case MVM_ARRAY_I16:
+        case MVM_ARRAY_I8:
+            return MVM_reg_int64;
+            break;
+        case MVM_ARRAY_N64:
+        case MVM_ARRAY_N32:
+            return MVM_reg_num64;
+            break;
+        case MVM_ARRAY_U64:
+        case MVM_ARRAY_U32:
+        case MVM_ARRAY_U16:
+        case MVM_ARRAY_U8:
+            return MVM_reg_uint64;
+            break;
+        default:
+            abort(); /* never reached, silence compiler warnings */
+    }
+}
+
 static void copy_elements(MVMThreadContext *tc, MVMObject *src, MVMObject *dest, MVMint64 s_offset, MVMint64 d_offset, MVMint64 elems) {
     MVMArrayBody     *s_body      = (MVMArrayBody *)OBJECT_BODY(src);
     MVMArrayBody     *d_body      = (MVMArrayBody *)OBJECT_BODY(dest);
@@ -816,7 +845,6 @@ static void copy_elements(MVMThreadContext *tc, MVMObject *src, MVMObject *dest,
 
     if (elems > 0) {
         MVMint64  i;
-        MVMuint16 kind;
         MVMuint8 d_needs_barrier = dest->header.flags2 & MVM_CF_SECOND_GEN;
         if (s_repr_data
                 && s_repr_data->slot_type == d_repr_data->slot_type
@@ -832,36 +860,13 @@ static void copy_elements(MVMThreadContext *tc, MVMObject *src, MVMObject *dest,
             );
         }
         else {
-            switch (d_repr_data->slot_type) {
-                case MVM_ARRAY_OBJ:
-                    kind = MVM_reg_obj;
-                    break;
-                case MVM_ARRAY_STR:
-                    kind = MVM_reg_str;
-                    break;
-                case MVM_ARRAY_I64:
-                case MVM_ARRAY_I32:
-                case MVM_ARRAY_I16:
-                case MVM_ARRAY_I8:
-                    kind = MVM_reg_int64;
-                    break;
-                case MVM_ARRAY_N64:
-                case MVM_ARRAY_N32:
-                    kind = MVM_reg_num64;
-                    break;
-                case MVM_ARRAY_U64:
-                case MVM_ARRAY_U32:
-                case MVM_ARRAY_U16:
-                case MVM_ARRAY_U8:
-                    kind = MVM_reg_int64;
-                    break;
-                default:
-                    abort(); /* never reached, silence compiler warnings */
-            }
+            MVMuint16 target_kind = slot_type_to_kind(d_repr_data->slot_type);
+            MVMuint16 source_kind = slot_type_to_kind(s_repr_data->slot_type);
             for (i = 0; i < elems; i++) {
                 MVMRegister to_copy;
-                REPR(src)->pos_funcs.at_pos(tc, STABLE(src), src, s_body, s_offset + i, &to_copy, kind);
-                MVM_VMArray_bind_pos(tc, STABLE(dest), dest, d_body, d_offset + i, to_copy, kind);
+                REPR(src)->pos_funcs.at_pos(tc, STABLE(src), src, s_body, s_offset + i, &to_copy, source_kind);
+                /* actually should coerce between source_kind and target_kind here */
+                MVM_VMArray_bind_pos(tc, STABLE(dest), dest, d_body, d_offset + i, to_copy, target_kind);
             }
         }
     }
@@ -910,7 +915,7 @@ static void write_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void
     memcpy(body->slots.u8 + (start + offset) * repr_data->elem_size, from, count);
 }
 
-MVMint64 read_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 offset, MVMuint64 count) {
+static MVMint64 read_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 offset, MVMuint64 count) {
     MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
     MVMArrayBody     *body      = (MVMArrayBody *)data;
     MVMint64 start = body->start;
@@ -1077,7 +1082,7 @@ static MVMStorageSpec get_elem_storage_spec(MVMThreadContext *tc, MVMSTable *st)
         case MVM_ARRAY_U16:
         case MVM_ARRAY_U8:
             spec.inlineable      = MVM_STORAGE_SPEC_INLINED;
-            spec.boxed_primitive = MVM_STORAGE_SPEC_BP_INT;
+            spec.boxed_primitive = MVM_STORAGE_SPEC_BP_UINT64;
             spec.can_box         = MVM_STORAGE_SPEC_CAN_BOX_INT;
             spec.is_unsigned     = 1;
             break;
@@ -1122,6 +1127,7 @@ static AO_t * pos_as_atomic_multidim(MVMThreadContext *tc, MVMSTable *st, MVMObj
 /* Compose the representation. */
 static void spec_to_repr_data(MVMThreadContext *tc, MVMArrayREPRData *repr_data, const MVMStorageSpec *spec) {
     switch (spec->boxed_primitive) {
+        case MVM_STORAGE_SPEC_BP_UINT64:
         case MVM_STORAGE_SPEC_BP_INT:
             if (spec->is_unsigned) {
                 switch (spec->bits) {
@@ -1502,6 +1508,14 @@ void *MVM_VMArray_find_fast_impl_for_jit(MVMThreadContext *tc, MVMSTable *st, MV
                 return vmarray_at_pos_int64;
             }
             break;
+        case MVM_OP_atpos_u:
+            if (kind != MVM_reg_uint64) {
+                return NULL;
+            }
+            if (repr_data->slot_type == MVM_ARRAY_U64) {
+                return vmarray_at_pos_int64;
+            }
+            break;
         case MVM_OP_bindpos_i:
             if (kind != MVM_reg_int64) {
                 return NULL;
@@ -1510,6 +1524,14 @@ void *MVM_VMArray_find_fast_impl_for_jit(MVMThreadContext *tc, MVMSTable *st, MV
                 return vmarray_bind_pos_int64;
             }
             break;
+        case MVM_OP_bindpos_u:
+            if (kind != MVM_reg_uint64) {
+                return NULL;
+            }
+            if (repr_data->slot_type == MVM_ARRAY_U64) {
+                return vmarray_bind_pos_int64;
+            }
+            break;
         default:
             return NULL;
     }
