@@ -162,8 +162,9 @@ MVMint64 MVM_sc_find_code_idx(MVMThreadContext *tc, MVMSerializationContext *sc,
     roots = sc->body->root_codes;
     count = MVM_repr_elems(tc, roots);
     for (i = 0; i < count; i++) {
-        MVMObject *test = MVM_repr_at_pos_o(tc, roots, i);
-        if (test == obj)
+        MVMRegister test;
+        MVM_VMArray_at_pos(tc, STABLE(roots), roots, OBJECT_BODY(roots), i, &test, MVM_reg_obj);
+        if (test.o == obj)
             return i;
     }
 
@@ -245,6 +246,21 @@ MVMObject * MVM_sc_try_get_object(MVMThreadContext *tc, MVMSerializationContext
         return NULL;
 }
 
+/* Given an SC, an index, and an object, store the object at that index and update
+ * its SC index to match. */
+void MVM_sc_set_object_op(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *obj) {
+    if (REPR(sc)->ID != MVM_REPR_ID_SCRef)
+        MVM_exception_throw_adhoc(tc,
+            "Must provide an SCRef operand to scsetobj");
+    MVM_sc_set_object(tc, sc, idx, obj);
+    if (MVM_sc_get_stable_sc(tc, STABLE(obj)) == NULL) {
+        /* Need to claim the SC also; typical case for new type objects. */
+        MVMSTable *st = STABLE(obj);
+        MVM_sc_set_stable_sc(tc, st, (MVMSerializationContext *)sc);
+        MVM_sc_push_stable(tc, (MVMSerializationContext *)sc, st);
+    }
+}
+
 /* Given an SC, an index, and an object, store the object at that index and update
  * its SC index to match. */
 void MVM_sc_set_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *obj) {
@@ -355,6 +371,14 @@ MVMObject * MVM_sc_get_code(MVMThreadContext *tc, MVMSerializationContext *sc, M
     }
 }
 
+/* Given an SC, an index and a code ref, store it and the index. */
+void MVM_sc_set_code_op(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMObject *code) {
+    if (REPR(sc)->ID != MVM_REPR_ID_SCRef)
+        MVM_exception_throw_adhoc(tc,
+            "Must provide an SCRef operand to scsetcode");
+    MVM_sc_set_obj_sc(tc, code, sc);
+    MVM_sc_set_code(tc, sc, idx, code);
+}
 /* Resolves an SC handle using the SC weakhash. */
 MVMSerializationContext * MVM_sc_find_by_handle(MVMThreadContext *tc, MVMString *handle) {
     uv_mutex_lock(&tc->instance->mutex_sc_registry);
