@@ -11,13 +11,10 @@ MVMint64 MVM_coerce_istrue_s(MVMThreadContext *tc, MVMString *str) {
 }
 
 /* ui64toa and i64toa
- * Copyright(c) 2014-2016 Milo Yip (miloyip@gmail.com)
- * https://github.com/miloyip/itoa-benchmark
- * With minor modifications.
+MIT License
 
- Copyright (C) 2014 Milo Yip
-
-MIT License:
+Copyright (c) 2017 James Edward Anhalt III - https://github.com/jeaiii/itoa
+with minor modifications for formatting and so it will compile as C code
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,43 +23,68 @@ to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-    THE SOFTWARE. */
-static char * u64toa_naive_worker(uint64_t value, char* buffer) {
-    char temp[20];
-    char *p = temp;
-    do {
-        *p++ = (char)(value % 10) + '0';
-        value /= 10;
-    } while (value > 0);
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+SOFTWARE. */
 
-    do {
-        *buffer++ = *--p;
-    } while (p != temp);
+#define W(N, I) b[N] = (char)(I) + '0'
+#define A(N) t = ((uint64_t)(1) << (32 + N / 5 * N * 53 / 16)) / (uint32_t)(1e##N) + 1 - N / 9, t *= u, t >>= N / 5 * N * 53 / 16, t += N / 5 * 4, W(0, t >> 32)
+#define D(N) t = (uint64_t)(10) * (uint32_t)(t), W(N, t >> 32)
 
-    return buffer;
-}
-static size_t i64toa_naive(int64_t value, char* buffer) {
-    uint64_t u = value;
-    char *buf = buffer;
-    if (value < 0) {
-        *buf++ = '-';
-        u = ~u + 1;
+#define L0 W(0, u)
+#define L1 A(1), D(1)
+#define L2 A(2), D(1), D(2)
+#define L3 A(3), D(1), D(2), D(3)
+#define L4 A(4), D(1), D(2), D(3), D(4)
+#define L5 A(5), D(1), D(2), D(3), D(4), D(5)
+#define L6 A(6), D(1), D(2), D(3), D(4), D(5), D(6)
+#define L7 A(7), D(1), D(2), D(3), D(4), D(5), D(6), D(7)
+#define L8 A(8), D(1), D(2), D(3), D(4), D(5), D(6), D(7), D(8)
+#define L9 A(9), D(1), D(2), D(3), D(4), D(5), D(6), D(7), D(8), D(9)
+
+#define LN(N) (L##N, b += N + 1)
+#define LZ LN
+
+#define LG(F) (u<100 ? u<10 ? F(0) : F(1) : u<1000000 ? u<10000 ? u<1000 ? F(2) : F(3) : u<100000 ? F(4) : F(5) : u<100000000 ? u<10000000 ? F(6) : F(7) : u<1000000000 ? F(8) : F(9))
+
+static char * u64toa_jeaiii(uint64_t n, char* b) {
+    uint32_t u;
+    uint64_t t;
+
+    if ((uint32_t)(n >> 32) == 0)
+        return u = (uint32_t)(n), LG(LZ);
+
+    uint64_t a = n / 100000000;
+
+    if ((uint32_t)(a >> 32) == 0) {
+        u = (uint32_t)(a);
+        LG(LN);
     }
-    return u64toa_naive_worker(u, buf) - buffer;
+    else {
+        u = (uint32_t)(a / 100000000);
+        LG(LN);
+        u = a % 100000000;
+        LN(7);
+    }
+
+    u = n % 100000000;
+    return LZ(7);
 }
+
+static char * i64toa_jeaiii(int64_t i, char* b) {
+    uint64_t n = i < 0 ? *b++ = '-', 0 - (uint64_t)(i) : (uint64_t)i;
+    return u64toa_jeaiii(n, b);
+}
+
 /* End code */
-static size_t u64toa_naive(uint64_t value, char* buffer) {
-    return u64toa_naive_worker(value, buffer) - buffer;
-}
+
 MVMString * MVM_coerce_i_s(MVMThreadContext *tc, MVMint64 i) {
     char buffer[20];
     int len;
@@ -74,7 +96,7 @@ MVMString * MVM_coerce_i_s(MVMThreadContext *tc, MVMint64 i) {
             return cached;
     }
     /* Otherwise, need to do the work; cache it if in range. */
-    len = i64toa_naive(i, buffer);
+    len = i64toa_jeaiii(i, buffer) - buffer;
     if (0 <= len) {
         MVMString *result = NULL;
         MVMGrapheme8 *blob = MVM_malloc(len);
@@ -100,7 +122,7 @@ MVMString * MVM_coerce_u_s(MVMThreadContext *tc, MVMuint64 i) {
             return cached;
     }
     /* Otherwise, need to do the work; cache it if in range. */
-    len = u64toa_naive(i, buffer);
+    len = u64toa_jeaiii(i, buffer) - buffer;
     if (0 <= len) {
         MVMString *result = NULL;
         MVMGrapheme8 *blob = MVM_malloc(len);
@@ -310,11 +332,162 @@ MVMString * MVM_coerce_n_s(MVMThreadContext *tc, MVMnum64 n) {
     return MVM_string_ascii_from_buf_nocheck(tc, blob, orig_len);
 }
 
-MVMint64 MVM_coerce_s_i(MVMThreadContext *tc, MVMString *s) {
-    char     *enc = MVM_string_ascii_encode_any(tc, s);
-    MVMint64  i   = strtoll(enc, NULL, 10);
-    MVM_free(enc);
-    return i;
+MVMint64 MVM_coerce_s_i(MVMThreadContext *tc, MVMString *str) {
+    MVMStringIndex strgraphs = MVM_string_graphs(tc, str);
+    MVMint64       result = 0;
+    MVMint32       any = 0, negative = 0;
+
+    signed long long cutoff;
+    MVMint32  cutlim;
+
+    if (!strgraphs)
+        return result;
+
+/*-
+ * Copyright (c) 1990 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * copied from https://github.com/gcc-mirror/gcc/blob/0c0f453c4af4880c522c8472c33eef42bee9eda1/libiberty/strtoll.c
+ * with minor modifications to simplify and work in MoarVM
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. [rescinded 22 July 1999]
+ * 4. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+    if (str->body.storage_type == MVM_STRING_GRAPHEME_ASCII) {
+        const MVMGraphemeASCII *s = str->body.storage.blob_ascii;
+        MVMStringIndex i = 0;
+        MVMGraphemeASCII c;
+        do {
+            c = *s++;
+            ++i;
+            /* The original C code was processing a NUL terminated string, hence
+             * it could always be sure that it could read a value into c for
+             * which isspace(c) was false, and so for a string entirely
+             * whitespace this loop would terminate with c == 0 (without reading
+             * beyond the end) and the code below would behave correctly too.
+             *
+             * Instead we can use the length test to terminate the loop
+             * unconditionally just after we read the last character. This would
+             * be for strings which are entirely whitespace - eg " ", "  " etc.
+             * They all return 0, and we don't need to optimise their handling,
+             * so we simply drop through into the rest of the code with c == ' '
+             * for them. */
+            /* isspace(...) is any of "\t\n\v\f\r ", ie [9 .. 13, 32] */
+        } while (i != strgraphs && (c == ' ' || (c >= '\t' && c <= '\r')));
+
+        /* `i` counts how many octets we have read. Hence `i == strgraphs` at
+         * the point where `c` holds the final ASCII character of the string,
+         * and there is no more to read. */
+        if (c == '-') {
+            negative = 1;
+            if (i++ == strgraphs)
+                return 0;
+            c = *s++;
+        } else if (c == '+') {
+            if (i++ == strgraphs)
+                return 0;
+            c = *s++;
+        }
+
+        cutoff = negative ? -(unsigned long long)LLONG_MIN : LLONG_MAX;
+        cutlim = cutoff % (unsigned long long)10;
+        cutoff /= (unsigned long long)10;
+
+        while (1) {
+            if (c >= '0' && c <= '9')
+                c -= '0';
+            else
+                break;
+
+            if (any < 0 || result > cutoff || (result == cutoff && c > cutlim))
+                any = -1;
+            else {
+                any = 1;
+                result *= 10;
+                result += c;
+            }
+            if (i++ == strgraphs)
+                break;
+            c = *s++;
+        }
+
+        if (any < 0)
+            result = negative ? LLONG_MIN : LLONG_MAX;
+        else if (negative)
+            result = -result;
+    }
+    else {
+        MVMCodepointIter ci;
+        MVM_string_ci_init(tc, &ci, str, 0, 0);
+        MVMCodepoint ord;
+
+        do {
+            ord = MVM_string_ci_get_codepoint(tc, &ci);
+        } while ((ord == ' ' || (ord >= '\t' && ord <= '\r')) && MVM_string_ci_has_more(tc, &ci));
+
+        if (ord == '-') {
+            negative = 1;
+            if (!MVM_string_ci_has_more(tc, &ci))
+                return 0;
+            ord = MVM_string_ci_get_codepoint(tc, &ci);
+        }
+        else if (ord == '+') {
+            if (!MVM_string_ci_has_more(tc, &ci))
+                return 0;
+            ord = MVM_string_ci_get_codepoint(tc, &ci);
+        }
+
+        cutoff = negative ? -(unsigned long long)LLONG_MIN : LLONG_MAX;
+        cutlim = cutoff % (unsigned long long)10;
+        cutoff /= (unsigned long long)10;
+
+        while (1) {
+            if (ord >= '0' && ord <= '9')
+                ord -= '0';
+            else
+                break;
+
+            if (any < 0 || result > cutoff || (result == cutoff && ord > cutlim))
+                any = -1;
+            else {
+                any = 1;
+                result *= 10;
+                result += ord;
+            }
+            if (!MVM_string_ci_has_more(tc, &ci))
+                break;
+            ord = MVM_string_ci_get_codepoint(tc, &ci);
+        };
+
+        if (any < 0)
+            result = negative ? LLONG_MIN : LLONG_MAX;
+        else if (negative)
+            result = -result;
+    }
+
+    return result;
 }
 
 MVMint64 MVM_coerce_simple_intify(MVMThreadContext *tc, MVMObject *obj) {
@@ -344,10 +517,10 @@ MVMint64 MVM_coerce_simple_intify(MVMThreadContext *tc, MVMObject *obj) {
 MVMObject * MVM_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *str, MVMint64 offset, MVMint64 flag) {
     MVMObject *result;
     MVMint64 zvalue = 0;
-    MVMint64 zbase  = 1;
     MVMint64 chars  = MVM_string_graphs(tc, str);
     MVMint64 value  = zvalue;
-    MVMint64 base   = zbase;
+    MVMuint32 chars_converted = 0;
+    MVMuint32 chars_really_converted = chars_converted;
     MVMint64   pos  = -1;
     MVMuint16  neg  = 0;
     MVMint64   ch;
@@ -384,9 +557,9 @@ MVMObject * MVM_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *str, MVMi
         else break;
         if (ch >= radix) break;
         zvalue = zvalue * radix + ch;
-        zbase = zbase * radix;
         offset++; pos = offset;
-        if (ch != 0 || !(flag & 0x04)) { value=zvalue; base=zbase; }
+        chars_converted++;
+        if (ch != 0 || !(flag & 0x04)) { value=zvalue; chars_really_converted = chars_converted; }
         if (offset >= chars) break;
         ch = MVM_string_get_grapheme_at_nocheck(tc, str, offset);
         if (ch != '_') continue;
@@ -404,7 +577,7 @@ MVMObject * MVM_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *str, MVMi
         MVMROOT(tc, box_type, {
             MVMObject *boxed = MVM_repr_box_int(tc, box_type, value);
             MVM_repr_push_o(tc, result, boxed);
-            boxed = MVM_repr_box_int(tc, box_type, base);
+            boxed = MVM_repr_box_int(tc, box_type, chars_really_converted);
             MVM_repr_push_o(tc, result, boxed);
             boxed = MVM_repr_box_int(tc, box_type, pos);
             MVM_repr_push_o(tc, result, boxed);
