@@ -10,7 +10,7 @@ static int crash_on_error = 0;
 /* Function for getting effective (specialized or not) frame handlers. */
 MVM_STATIC_INLINE MVMFrameHandler * MVM_frame_effective_handlers(MVMFrame *f) {
     MVMSpeshCandidate *spesh_cand = f->spesh_cand;
-    return spesh_cand ? spesh_cand->handlers : f->static_info->body.handlers;
+    return spesh_cand ? spesh_cand->body.handlers : f->static_info->body.handlers;
 }
 
 /* Maps ID of exception category to its name. */
@@ -100,10 +100,10 @@ static MVMint32 search_frame_handlers_dyn(MVMThreadContext *tc, MVMFrame *f,
                                           MVMuint32 cat, MVMObject *payload,
                                           LocatedHandler *lh) {
     MVMuint32  i;
-    if (f->spesh_cand && f->spesh_cand->jitcode && f->jit_entry_label) {
-        MVMJitCode *jitcode = f->spesh_cand->jitcode;
+    if (f->spesh_cand && f->spesh_cand->body.jitcode && f->jit_entry_label) {
+        MVMJitCode *jitcode = f->spesh_cand->body.jitcode;
         void *current_position = MVM_jit_code_get_current_position(tc, jitcode, f);
-        MVMJitHandler    *jhs = f->spesh_cand->jitcode->handlers;
+        MVMJitHandler    *jhs = f->spesh_cand->body.jitcode->handlers;
         MVMFrameHandler  *fhs = MVM_frame_effective_handlers(f);
         for (i = MVM_jit_code_get_active_handlers(tc, jitcode, current_position, 0);
              i < jitcode->num_handlers;
@@ -117,7 +117,7 @@ static MVMint32 search_frame_handlers_dyn(MVMThreadContext *tc, MVMFrame *f,
         }
     } else {
         MVMuint32 num_handlers = f->spesh_cand
-            ? f->spesh_cand->num_handlers
+            ? f->spesh_cand->body.num_handlers
             : f->static_info->body.num_handlers;
         MVMuint32 pc;
         if (f == tc->cur_frame)
@@ -175,8 +175,8 @@ static MVMint32 search_frame_handlers_lex(MVMThreadContext *tc, MVMFrame *f,
     MVMuint32 i;
     MVMuint32 skipping = *skip_first_inlinee;
     MVMFrameHandler *fhs = MVM_frame_effective_handlers(f);
-    if (f->spesh_cand && f->spesh_cand->jitcode && f->jit_entry_label) {
-        MVMJitCode *jitcode = f->spesh_cand->jitcode;
+    if (f->spesh_cand && f->spesh_cand->body.jitcode && f->jit_entry_label) {
+        MVMJitCode *jitcode = f->spesh_cand->body.jitcode;
         void *current_position = MVM_jit_code_get_current_position(tc, jitcode, f);
         MVMJitHandler    *jhs = jitcode->handlers;
         for (i = MVM_jit_code_get_active_handlers(tc, jitcode, current_position, 0);
@@ -191,7 +191,7 @@ static MVMint32 search_frame_handlers_lex(MVMThreadContext *tc, MVMFrame *f,
                     *skip_first_inlinee = 0;
                 }
                 else {
-                    MVMuint16 cr_reg = f->spesh_cand->inlines[fh->inlinee].code_ref_reg;
+                    MVMuint16 cr_reg = f->spesh_cand->body.inlines[fh->inlinee].code_ref_reg;
                     MVMFrame *inline_outer = ((MVMCode *)f->work[cr_reg].o)->body.outer;
                     if (inline_outer == f) {
                         skip_all_inlinees = 1;
@@ -215,7 +215,7 @@ static MVMint32 search_frame_handlers_lex(MVMThreadContext *tc, MVMFrame *f,
     }
     else {
         MVMuint32 num_handlers = f->spesh_cand
-            ? f->spesh_cand->num_handlers
+            ? f->spesh_cand->body.num_handlers
             : f->static_info->body.num_handlers;
         MVMuint32 pc;
         if (f == tc->cur_frame)
@@ -233,7 +233,7 @@ static MVMint32 search_frame_handlers_lex(MVMThreadContext *tc, MVMFrame *f,
                         *skip_first_inlinee = 0;
                     }
                     else {
-                        MVMuint16 cr_reg = f->spesh_cand->inlines[fh->inlinee].code_ref_reg;
+                        MVMuint16 cr_reg = f->spesh_cand->body.inlines[fh->inlinee].code_ref_reg;
                         MVMFrame *inline_outer = ((MVMCode *)f->work[cr_reg].o)->body.outer;
                         if (inline_outer == f) {
                             skip_all_inlinees = 1;
@@ -348,8 +348,8 @@ static void run_handler(MVMThreadContext *tc, LocatedHandler lh, MVMObject *ex_o
 
     case MVM_EX_ACTION_GOTO:
         if (lh.jit_handler) {
-            void **labels = lh.frame->spesh_cand->jitcode->labels;
-            MVMuint8  *pc = lh.frame->spesh_cand->jitcode->bytecode;
+            void **labels = lh.frame->spesh_cand->body.jitcode->labels;
+            MVMuint8  *pc = lh.frame->spesh_cand->body.jitcode->bytecode;
             MVM_frame_unwind_to(tc, lh.frame, pc, 0, NULL, labels[lh.jit_handler->goto_label]);
         } else {
             MVM_frame_unwind_to(tc, lh.frame, NULL, lh.handler->goto_offset, NULL, NULL);
@@ -359,12 +359,10 @@ static void run_handler(MVMThreadContext *tc, LocatedHandler lh, MVMObject *ex_o
 
     case MVM_EX_ACTION_INVOKE: {
         /* Create active handler record. */
-        MVMActiveHandler *ah = MVM_malloc(sizeof(MVMActiveHandler));
-        MVMFrame *cur_frame = tc->cur_frame;
-        MVMFrame *pres_frame;
-        MVMObject *handler_code;
+        MVMActiveHandler *ah = MVM_fixed_size_alloc(tc, tc->instance->fsa, sizeof(MVMActiveHandler));
 
         /* Ensure we have an exception object. */
+        MVMFrame *cur_frame = tc->cur_frame;
         if (ex_obj == NULL) {
             MVMROOT3(tc, cur_frame, lh.frame, payload, {
                 ex_obj = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTException);
@@ -374,7 +372,7 @@ static void run_handler(MVMThreadContext *tc, LocatedHandler lh, MVMObject *ex_o
         }
 
         /* Preserve the frame caller chain, so we can do backtraces. */
-        pres_frame = ((MVMException *)ex_obj)->body.origin;
+        MVMFrame *pres_frame = ((MVMException *)ex_obj)->body.origin;
         while (pres_frame) {
             MVMFrameExtra *extra = MVM_frame_extra(tc, pres_frame);
             extra->caller_info_needed = 1;
@@ -382,7 +380,9 @@ static void run_handler(MVMThreadContext *tc, LocatedHandler lh, MVMObject *ex_o
         }
 
         /* Find frame to invoke. */
-        handler_code = MVM_frame_find_invokee(tc, lh.frame->work[lh.handler->block_reg].o, NULL);
+        MVMObject *handler_code = lh.frame->work[lh.handler->block_reg].o;
+        if (!MVM_code_iscode(tc, handler_code))
+            MVM_oops(tc, "Exception handler must be a VM code handle");
 
         /* Install active handler record. */
         ah->frame           = lh.frame;
@@ -397,12 +397,13 @@ static void run_handler(MVMThreadContext *tc, LocatedHandler lh, MVMObject *ex_o
         cur_frame->return_value = (MVMRegister *)&tc->last_handler_result;
         cur_frame->return_type = MVM_RETURN_OBJ;
         cur_frame->return_address = *(tc->interp_cur_op);
-        MVM_frame_special_return(tc, cur_frame, unwind_after_handler, cleanup_active_handler,
-            ah, NULL);
+        MVMActiveHandler **sr = MVM_callstack_allocate_special_return(tc,
+                unwind_after_handler, cleanup_active_handler,
+                NULL, sizeof(MVMActiveHandler *));
+        *sr = ah;
 
         /* Invoke the handler frame and return to runloop. */
-        STABLE(handler_code)->invoke(tc, handler_code, MVM_callsite_get_common(tc, MVM_CALLSITE_ID_NULL_ARGS),
-                                     cur_frame->args);
+        MVM_frame_dispatch_zero_args(tc, (MVMCode *)handler_code);
         break;
     }
     default:
@@ -420,7 +421,7 @@ static void unwind_after_handler(MVMThreadContext *tc, void *sr_data) {
 
     /* Get active handler; sanity check (though it's possible other cases
      * should be supported). */
-    MVMActiveHandler *ah = (MVMActiveHandler *)sr_data;
+    MVMActiveHandler *ah = *((MVMActiveHandler **)sr_data);
     if (tc->active_handlers != ah)
         MVM_panic(1, "Trying to unwind from wrong handler");
 
@@ -428,9 +429,9 @@ static void unwind_after_handler(MVMThreadContext *tc, void *sr_data) {
     frame       = ah->frame;
     exception   = (MVMException *)ah->ex_obj;
     if (ah->jit_handler) {
-        void **labels = frame->spesh_cand->jitcode->labels;
+        void **labels = frame->spesh_cand->body.jitcode->labels;
         jit_return_label = labels[ah->jit_handler->goto_label];
-        abs_address = frame->spesh_cand->jitcode->bytecode;
+        abs_address = frame->spesh_cand->body.jitcode->bytecode;
         goto_offset = 0;
     }
     else {
@@ -440,7 +441,7 @@ static void unwind_after_handler(MVMThreadContext *tc, void *sr_data) {
     }
     /* Clean up. */
     tc->active_handlers = ah->next_handler;
-    MVM_free(ah);
+    MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMActiveHandler), ah);
 
     /* Do the unwinding as needed. */
     if (exception && exception->body.return_after_unwind) {
@@ -456,13 +457,13 @@ static void unwind_after_handler(MVMThreadContext *tc, void *sr_data) {
 static void cleanup_active_handler(MVMThreadContext *tc, void *sr_data) {
     /* Get active handler; sanity check (though it's possible other cases
      * should be supported). */
-    MVMActiveHandler *ah = (MVMActiveHandler *)sr_data;
+    MVMActiveHandler *ah = *((MVMActiveHandler **)sr_data);
     if (tc->active_handlers != ah)
         MVM_panic(1, "Trying to unwind over wrong handler");
 
     /* Clean up. */
     tc->active_handlers = ah->next_handler;
-    MVM_free(ah);
+    MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMActiveHandler), ah);
 }
 
 char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame,
@@ -514,6 +515,7 @@ char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame,
 /* Returns a list of hashes containing file, line, sub and annotations. */
 MVMObject * MVM_exception_backtrace(MVMThreadContext *tc, MVMObject *ex_obj) {
     MVMFrame *cur_frame;
+    MVMSpeshFrameWalker fw;
     MVMObject *arr = NULL, *annotations = NULL, *row = NULL, *value = NULL;
     MVMuint32 count = 0;
     MVMString *k_file = NULL, *k_line = NULL, *k_sub = NULL, *k_anno = NULL;
@@ -523,6 +525,10 @@ MVMObject * MVM_exception_backtrace(MVMThreadContext *tc, MVMObject *ex_obj) {
         cur_frame = ((MVMException *)ex_obj)->body.origin;
         throw_address = ((MVMException *)ex_obj)->body.throw_address;
     }
+    else if (ex_obj == tc->instance->VMNull) {
+        cur_frame = tc->cur_frame;
+        throw_address = *tc->interp_cur_op;
+    }
     else {
         MVM_exception_throw_adhoc(tc, "Op 'backtrace' needs an exception object");
     }
@@ -544,46 +550,50 @@ MVMObject * MVM_exception_backtrace(MVMThreadContext *tc, MVMObject *ex_obj) {
 
     arr = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTArray);
 
-    while (cur_frame != NULL) {
-        MVMuint8             *cur_op = count ? cur_frame->return_address : throw_address;
-        MVMuint32             offset = cur_op - MVM_frame_effective_bytecode(cur_frame);
-        MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc, &cur_frame->static_info->body,
-                                            offset > 0 ? offset - 1 : 0);
-        MVMuint32             fshi   = annot ? (MVMint32)annot->filename_string_heap_index : -1;
-        char            *line_number = MVM_malloc(16);
-        MVMString      *filename_str;
-        snprintf(line_number, 16, "%d", annot ? annot->line_number : 1);
+    if (cur_frame) {
+        MVM_spesh_frame_walker_init(tc, &fw, cur_frame, 0);
+        MVM_spesh_frame_walker_next(tc, &fw); // Go to first inline
 
-        /* annotations hash will contain "file" and "line" */
-        annotations = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTHash);
+        do {
+            cur_frame = MVM_spesh_frame_walker_current_frame(tc, &fw);
+            MVMuint8             *cur_op = count ? cur_frame->return_address : throw_address;
+            MVMuint32             offset = cur_op - MVM_frame_effective_bytecode(cur_frame);
+            MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc, &cur_frame->static_info->body,
+                                                offset > 0 ? offset - 1 : 0);
+            MVMuint32             fshi   = annot ? (MVMint32)annot->filename_string_heap_index : -1;
+            char         line_number[11] = {0}; // 11 == 10 (max # of decimal digits in an MVMuint32) + 1 (NULL terminator)
+            MVMString      *filename_str;
+            snprintf(line_number, 11, "%d", annot ? annot->line_number : 1);
 
-        /* file */
-        filename_str = annot && fshi < cur_frame->static_info->body.cu->body.num_strings
-             ? MVM_cu_string(tc, cur_frame->static_info->body.cu, fshi)
-             : cur_frame->static_info->body.cu->body.filename;
-        value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type,
-            filename_str ? filename_str : tc->instance->str_consts.empty);
-        MVM_repr_bind_key_o(tc, annotations, k_file, value);
+            /* annotations hash will contain "file" and "line" */
+            annotations = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTHash);
 
-        /* line */
-        value = (MVMObject *)MVM_string_ascii_decode_nt(tc, tc->instance->VMString, line_number);
-        value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type, (MVMString *)value);
-        MVM_repr_bind_key_o(tc, annotations, k_line, value);
-        MVM_free(line_number);
+            /* file */
+            filename_str = annot && fshi < cur_frame->static_info->body.cu->body.num_strings
+                 ? MVM_cu_string(tc, cur_frame->static_info->body.cu, fshi)
+                 : cur_frame->static_info->body.cu->body.filename;
+            value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type,
+                filename_str ? filename_str : tc->instance->str_consts.empty);
+            MVM_repr_bind_key_o(tc, annotations, k_file, value);
 
-        /* row will contain "sub" and "annotations" */
-        row = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTHash);
-        MVM_repr_bind_key_o(tc, row, k_sub,
-            cur_frame->code_ref ? cur_frame->code_ref : tc->instance->VMNull);
-        MVM_repr_bind_key_o(tc, row, k_anno, annotations);
+            /* line */
+            value = (MVMObject *)MVM_string_ascii_decode_nt(tc, tc->instance->VMString, line_number);
+            value = MVM_repr_box_str(tc, MVM_hll_current(tc)->str_box_type, (MVMString *)value);
+            MVM_repr_bind_key_o(tc, annotations, k_line, value);
 
-        MVM_repr_push_o(tc, arr, row);
-        MVM_free(annot);
+            /* row will contain "sub" and "annotations" */
+            row = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTHash);
+            MVM_repr_bind_key_o(tc, row, k_sub,
+                cur_frame->code_ref ? cur_frame->code_ref : tc->instance->VMNull);
+            MVM_repr_bind_key_o(tc, row, k_anno, annotations);
 
-        cur_frame = cur_frame->caller;
-        while (cur_frame && cur_frame->static_info->body.is_thunk)
-            cur_frame = cur_frame->caller;
-        count++;
+            MVM_repr_push_o(tc, arr, row);
+            MVM_free(annot);
+
+            count++;
+        } while (MVM_spesh_frame_walker_move_caller_skip_thunks(tc, &fw));
+
+        MVM_spesh_frame_walker_cleanup(tc, &fw);
     }
 
     MVM_gc_root_temp_pop_n(tc, 9);
@@ -695,18 +705,17 @@ static void panic_unhandled_ex(MVMThreadContext *tc, MVMException *ex) {
  * a handler for unlocated lexical handlers. */
 static MVMint32 use_lexical_handler_hll_error(MVMThreadContext *tc, MVMuint8 mode) {
     return (mode == MVM_EX_THROW_LEX || mode == MVM_EX_THROW_LEX_CALLER) &&
-        !MVM_is_null(tc, MVM_hll_current(tc)->lexical_handler_not_found_error);
+        !MVM_is_null(tc, (MVMObject *)MVM_hll_current(tc)->lexical_handler_not_found_error);
 }
 
 /* Invokes the HLL's handler for unresolved lexical throws. */
 static void invoke_lexical_handler_hll_error(MVMThreadContext *tc, MVMint64 cat, LocatedHandler lh) {
-    MVMObject *handler = MVM_hll_current(tc)->lexical_handler_not_found_error;
-    MVMCallsite *callsite = MVM_callsite_get_common(tc, MVM_CALLSITE_ID_INT_INT);
-    handler = MVM_frame_find_invokee(tc, handler, NULL);
-    MVM_args_setup_thunk(tc, NULL, MVM_RETURN_VOID, callsite);
-    tc->cur_frame->args[0].i64 = cat;
-    tc->cur_frame->args[1].i64 = lh.handler_out_of_dynamic_scope;
-    STABLE(handler)->invoke(tc, handler, callsite, tc->cur_frame->args);
+    MVMCode *handler = MVM_hll_current(tc)->lexical_handler_not_found_error;
+    MVMCallStackArgsFromC *args_record = MVM_callstack_allocate_args_from_c(tc,
+            MVM_callsite_get_common(tc, MVM_CALLSITE_ID_INT_INT));
+    args_record->args.source[0].i64 = cat;
+    args_record->args.source[1].i64 = lh.handler_out_of_dynamic_scope;
+    MVM_frame_dispatch_from_c(tc, handler, args_record, NULL, MVM_RETURN_VOID);
 }
 
 /* Throws an exception by category, searching for a handler according to
@@ -762,7 +771,7 @@ void MVM_exception_throwobj(MVMThreadContext *tc, MVMuint8 mode, MVMObject *ex_o
         ex->body.resume_addr = *tc->interp_cur_op;
         /* Ensure that we store label where the JIT should return, if any */
         if (tc->jit_return_address != NULL) {
-            ex->body.jit_resume_label = MVM_jit_code_get_current_position(tc, tc->cur_frame->spesh_cand->jitcode, tc->cur_frame);
+            ex->body.jit_resume_label = MVM_jit_code_get_current_position(tc, tc->cur_frame->spesh_cand->body.jitcode, tc->cur_frame);
         }
     }
     lh = search_for_handler_from(tc, tc->cur_frame, mode, ex->body.category, ex->body.payload);
@@ -797,10 +806,7 @@ void MVM_exception_throwpayload(MVMThreadContext *tc, MVMuint8 mode, MVMuint32 c
 }
 
 void MVM_exception_resume(MVMThreadContext *tc, MVMObject *ex_obj) {
-    MVMException     *ex;
-    MVMFrame         *target;
-    MVMActiveHandler *ah;
-
+    MVMException *ex;
     if (IS_CONCRETE(ex_obj) && REPR(ex_obj)->ID == MVM_REPR_ID_MVMException)
         ex = (MVMException *)ex_obj;
     else
@@ -809,11 +815,9 @@ void MVM_exception_resume(MVMThreadContext *tc, MVMObject *ex_obj) {
     /* Check that everything is in place to do the resumption. */
     if (!ex->body.resume_addr)
         MVM_exception_throw_adhoc(tc, "This exception is not resumable");
-    target = ex->body.origin;
+    MVMFrame *target = ex->body.origin;
     if (!target)
         MVM_exception_throw_adhoc(tc, "This exception is not resumable");
-    if (!target->extra || target->extra->special_return != unwind_after_handler)
-        MVM_exception_throw_adhoc(tc, "This exception is not resumable");
     if (!in_caller_chain(tc, target))
         MVM_exception_throw_adhoc(tc, "Too late to resume this exception");
 
@@ -823,14 +827,6 @@ void MVM_exception_resume(MVMThreadContext *tc, MVMObject *ex_obj) {
     if (tc->active_handlers->ex_obj != ex_obj)
         MVM_exception_throw_adhoc(tc, "Can only resume the current exception");
 
-    /* Clear special return handler; we'll do its work here. */
-    MVM_frame_clear_special_return(tc, target);
-
-    /* Clear the current active handler. */
-    ah = tc->active_handlers;
-    tc->active_handlers = ah->next_handler;
-    MVM_free(ah);
-
     /* Unwind to the thrower of the exception; set PC and jit entry label. */
     MVM_frame_unwind_to(tc, target, ex->body.resume_addr, 0, NULL, ex->body.jit_resume_label);
 }
@@ -1014,6 +1010,23 @@ MVMObject * MVM_get_exception_payload(MVMThreadContext *tc, MVMObject *ex) {
     return result;
 }
 
+MVMString * MVM_get_exception_message(MVMThreadContext *tc, MVMObject *ex) {
+    if (IS_CONCRETE(ex) && REPR(ex)->ID == MVM_REPR_ID_MVMException)
+        return ((MVMException *)ex)->body.message;
+    else
+        MVM_exception_throw_adhoc(tc, "getexmessage needs a VMException, got %s (%s)", REPR(ex)->name, MVM_6model_get_debug_name(tc, ex));
+}
+
+void MVM_bind_exception_message(MVMThreadContext *tc, MVMObject *ex, MVMString *message) {
+    if (IS_CONCRETE(ex) && REPR(ex)->ID == MVM_REPR_ID_MVMException) {
+        MVM_ASSIGN_REF(tc, &(ex->header), ((MVMException *)ex)->body.message,
+            message);
+    }
+    else {
+        MVM_exception_throw_adhoc(tc, "bindexmessage needs a VMException, got %s (%s)", REPR(ex)->name, MVM_6model_get_debug_name(tc, ex));
+    }
+}
+
 void MVM_bind_exception_payload(MVMThreadContext *tc, MVMObject *ex, MVMObject *payload) {
     if (IS_CONCRETE(ex) && REPR(ex)->ID == MVM_REPR_ID_MVMException) {
         MVM_ASSIGN_REF(tc, &(ex->header), ((MVMException *)ex)->body.payload,
