@@ -290,15 +290,27 @@ char * MVM_nativecall_unmarshal_string(MVMThreadContext *tc, MVMObject *value, M
         char *str;
         switch (type & MVM_NATIVECALL_ARG_TYPE_MASK) {
             case MVM_NATIVECALL_ARG_ASCIISTR:
-                str = MVM_string_ascii_encode_any(tc, value_str);
+                str = MVM_string_ascii_encode_malloc(tc, value_str);
                 break;
             case MVM_NATIVECALL_ARG_UTF16STR:
-                str = MVM_string_utf16_encode(tc, value_str, 0);
+            {
+                MVMuint64 output_size;
+                /* This adds a (two byte) U-0000 at the end, but doesn't include
+                 * that code point in the returned output_size. */
+                char *temp = MVM_string_utf16_encode_substr(tc, value_str, &output_size, 0, -1, NULL, 0);
+                /* output_size is in 2 byte units, and doesn't include the
+                 * terminating U-0000, which was written. */
+                size_t octet_len = 2 * (output_size + 1);
+                str = malloc(octet_len);
+                memcpy(str, temp, octet_len);
+                MVM_free(temp);
                 break;
+            }
             default:
-                str = MVM_string_utf8_encode_C_string(tc, value_str);
+                str = MVM_string_utf8_encode_C_string_malloc(tc, value_str);
         }
 
+
         /* Set whether to free it or not. */
         if (free) {
             if (REPR(value)->ID == MVM_REPR_ID_MVMCStr)
@@ -509,6 +521,7 @@ MVMint8 MVM_nativecall_build(MVMThreadContext *tc, MVMObject *site, MVMString *l
     }
     body->arg_types = arg_types;
     body->arg_info  = arg_info;
+    MVM_barrier();
     body->num_args  = num_args; /* ensure we have properly populated arrays before setting num */
 
     /* Transform return argument type info a flag. */
