@@ -34,8 +34,8 @@ MVMThreadContext * MVM_tc_create(MVMThreadContext *parent, MVMInstance *instance
     /* The fixed size allocator also keeps pre-thread state. */
     MVM_fixed_size_create_thread(tc);
 
-    /* Allocate an initial call stack region for the thread. */
-    MVM_callstack_region_init(tc);
+    /* Allocate a call stack for the thread. */
+    MVM_callstack_init(tc);
 
     /* Initialize random number generator state. */
     MVM_proc_seed(tc, (MVM_platform_now() / 10000) * MVM_proc_getpid(tc));
@@ -52,22 +52,15 @@ MVMThreadContext * MVM_tc_create(MVMThreadContext *parent, MVMInstance *instance
             }
             MVM_free(tc->temp_bigints[i]);
             MVM_tc_destroy(tc);
-            MVM_exception_throw_adhoc(tc, "Error creating a temporary big integer: %s", mp_error_to_string(err));
+            MVM_exception_throw_adhoc(parent, "Error creating a temporary big integer: %s", mp_error_to_string(err));
         }
     }
 
-    /* Initialize frame sequence numbers */
-    tc->next_frame_nr = 0;
-    tc->current_frame_nr = 0;
-
     /* Initialize last_payload, so we can be sure it's never NULL and don't
      * need to check. */
     tc->last_payload = instance->VMNull;
 
-    /* Initialize plugin_guard_args so we never have to do a NULL check */
-    tc->plugin_guard_args = instance->VMNull;
-
-    /* Note that these two assignments above are repeated in
+    /* Note that the last assignment above is repeated in
      * MVM_6model_bootstrap because VMNull doesn't exist yet when the very
      * first tc is created. */
 
@@ -82,6 +75,14 @@ MVMThreadContext * MVM_tc_create(MVMThreadContext *parent, MVMInstance *instance
 void MVM_tc_destroy(MVMThreadContext *tc) {
     MVMint32 i;
 
+    /* If an exception handler calls nqp::exit, we don't unwind the stack and
+     * exception handlers aren't cleaned up yet */
+    while (tc->active_handlers) {
+        MVMActiveHandler *ah = tc->active_handlers;
+        tc->active_handlers = ah->next_handler;
+        MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMActiveHandler), ah);
+    }
+
     /* Free the native callback cache. Needs the fixed size allocator. */
     /* (currently not. but might if MVMStrHash moves internally to the FSA.) */
     MVM_str_hash_demolish(tc, &tc->native_callback_cache);
@@ -106,8 +107,8 @@ void MVM_tc_destroy(MVMThreadContext *tc) {
     /* Destory the per-thread fixed size allocator state. */
     MVM_fixed_size_destroy_thread(tc);
 
-    /* Destroy all callstack regions. */
-    MVM_callstack_region_destroy_all(tc);
+    /* Destroy the callstack, releasing all allocated memory. */
+    MVM_callstack_destroy(tc);
 
     /* Free the thread-specific storage */
     MVM_free(tc->gc_work);
