@@ -5,8 +5,10 @@
 /* pthread_setname_np only exists if we set _GNU_SOURCE extremely early.
  * We will need to be vgilant to not accidentally use gnu extensions in
  * other places without checking properly. */
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
+#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -14,6 +16,7 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 /* Standard integer types. */
 #include <platform/inttypes.h>
@@ -21,14 +24,58 @@
 /* platform-specific setjmp override */
 #include <platform/setjmp.h>
 
+#ifdef MVM_USE_MIMALLOC
+/* mimalloc needs to come early so other libs use it */
+#include <mimalloc.h>
+
+#define MVM_strdup mi_strdup
+#define MVM_strndup mi_strndup
+#else
+#define MVM_strdup strdup
+#define MVM_strndup strndup
+#endif
+
 /* libuv
  * must precede atomic_ops.h so we get the ordering of Winapi headers right
  */
 #include <uv.h>
 
+#ifdef MVM_USE_C11_ATOMICS
+#include <stdatomic.h>
+typedef atomic_uintptr_t AO_t;
+
+/* clang and gcc disagree on rvalue semantics of atomic types
+ * clang refuses to implicitly assign the value of an atomic variable to the
+ * regular non-atomic type. Hence we need the following for clang.
+ * Whereas gcc permits reading them as normal variables.
+ *
+ * We can also use `atomic_load_explicit` on gcc, which keeps our code simpler.
+ * Curiously the affect is different on different architectures. (This might be
+ * a gcc bug, or just ambiguity in the C standard).
+ * Using `atomic_load_explicit` instead of a simple read has these changes:
+ *
+ * * sparc64 removes `membar` instructions
+ * * arm64 changes from LFAR to LDR (LDAR has load-acquire semantics)
+ *
+ * but x86_64 and ppc64 are unchanged.
+ *
+ * suggesting that the former platforms are treating the implicit read as
+ * `memory_order_seq_cst` but the latter as `memory_order_relaxed`
+ *
+ * The latter is what we want, and is cheaper, so be explicit.
+ */
+#define AO_READ(v) atomic_load_explicit(&(v), memory_order_relaxed)
+
+/* clang also refuses to cast as (AO_t)(v), but doing this works for gcc and
+ * clang (and hopefully other compilers, when we get there) */
+#define AO_CAST(v) (uintptr_t)(v)
+#else
 /* libatomic_ops */
 #define AO_REQUIRE_CAS
 #include <atomic_ops.h>
+#define AO_READ(v) (v)
+#define AO_CAST(v) (AO_t)(v)
+#endif
 
 /* libffi or dynload/dyncall/dyncallback */
 #ifdef HAVE_LIBFFI
@@ -119,24 +166,31 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "core/ptr_hash_table.h"
 #include "core/uni_hash_table.h"
 #include "core/threadcontext.h"
+#include "disp/registry.h"
+#include "disp/boot.h"
+#include "disp/inline_cache.h"
 #include "core/instance.h"
 #include "core/interp.h"
+#include "core/fixedsizealloc.h"
 #include "core/callsite.h"
 #include "core/args.h"
+#include "disp/program.h"
+#include "disp/syscall.h"
+#include "disp/resume.h"
 #include "core/alloc.h"
 #include "core/frame.h"
 #include "core/callstack.h"
 #include "core/validation.h"
+#include "core/ops.h"
+#include "core/ext.h"
 #include "core/bytecode.h"
 #include "core/bytecodedump.h"
-#include "core/ops.h"
 #include "core/threads.h"
 #include "core/hll.h"
 #include "core/loadbytecode.h"
 #include "core/bitmap.h"
 #include "math/num.h"
 #include "core/coerce.h"
-#include "core/ext.h"
 #ifdef HAVE_LIBFFI
 #include "core/nativecall_libffi.h"
 #else
@@ -146,6 +200,7 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "core/dll.h"
 #include "core/continuation.h"
 #include "debug/debugserver.h"
+#include "spesh/pea.h"
 #include "6model/reprs.h"
 #include "6model/reprconv.h"
 #include "6model/bootstrap.h"
@@ -163,10 +218,9 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "core/regionalloc.h"
 #include "spesh/dump.h"
 #include "spesh/debug.h"
-#include "spesh/pea.h"
+#include "spesh/disp.h"
 #include "spesh/graph.h"
 #include "spesh/codegen.h"
-#include "spesh/candidate.h"
 #include "spesh/manipulate.h"
 #include "spesh/args.h"
 #include "spesh/usages.h"
@@ -185,7 +239,6 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "spesh/stats.h"
 #include "spesh/plan.h"
 #include "spesh/arg_guard.h"
-#include "spesh/plugin.h"
 #include "spesh/frame_walker.h"
 #include "strings/nfg.h"
 #include "strings/normalize.h"
@@ -197,7 +250,6 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "strings/utf16.h"
 #include "strings/iter.h"
 #include "strings/ops.h"
-#include "core/fixedsizealloc.h"
 #include "io/procops.h"
 #include "core/str_hash_table_funcs.h"
 #include "core/fixkey_hash_table_funcs.h"
@@ -268,6 +320,51 @@ MVM_PUBLIC int MVM_exepath(char* buffer, size_t* size);
 MVM_PUBLIC int MVM_set_std_handles_to_nul(void);
 #endif
 
+#ifdef MVM_USE_C11_ATOMICS
+
+#define MVM_incr(addr) atomic_fetch_add((volatile AO_t *)(addr), 1)
+#define MVM_decr(addr) atomic_fetch_sub((volatile AO_t *)(addr), 1)
+#define MVM_add(addr, add) atomic_fetch_add((volatile AO_t *)(addr), (add))
+
+/* Returns non-zero for success. Use for both AO_t numbers and pointers. */
+MVM_STATIC_INLINE int
+MVM_trycas_AO(volatile AO_t *addr, uintptr_t old, const uintptr_t new) {
+    return atomic_compare_exchange_strong(addr, &old, new);
+}
+#define MVM_trycas(addr, old, new) MVM_trycas_AO((volatile AO_t *)(addr), AO_CAST(old), AO_CAST(new))
+
+
+/* Returns the old value dereferenced at addr.
+ * Strictly, as libatomic_ops documents it:
+ *      Atomically compare *addr to old_val, and replace *addr by new_val
+ *      if the first comparison succeeds; returns the original value of *addr;
+ *       cannot fail spuriously.
+ */
+MVM_STATIC_INLINE uintptr_t
+MVM_cas(volatile AO_t *addr, uintptr_t old, const uintptr_t new) {
+    /* If *addr == old then { does exchange, returns true }
+     * else { writes old value to &old, returns false }
+     * Hence if exchange happens, we return the old value because C11 doesn't
+     * overwrite &old. If exchange doesn't happen, C11 does overwrite. */
+    atomic_compare_exchange_strong(addr, &old, new);
+    return old;
+}
+
+/* Returns the old pointer value dereferenced at addr. Provided for a tiny bit of type safety. */
+#define MVM_casptr(addr, old, new) ((void *)MVM_cas((AO_t *)(addr), (uintptr_t)(old), (uintptr_t)(new)))
+
+/* Full memory barrier. */
+#define MVM_barrier() atomic_thread_fence(memory_order_seq_cst)
+
+/* Need to use these to assign to or read from any memory locations on
+ * which the other atomic operation macros are used... */
+#define MVM_store(addr, new) atomic_store((volatile AO_t *)(addr), AO_CAST(new))
+#define MVM_load(addr) atomic_load((volatile AO_t *)(addr))
+
+#else
+
+/* libatomic_ops */
+
 /* Seems that both 32 and 64 bit sparc need this crutch */
 #if defined(__s390__) || defined(__sparc__)
 AO_t AO_fetch_compare_and_swap_emulation(volatile AO_t *addr, AO_t old_val, AO_t new_val);
@@ -296,3 +393,5 @@ AO_t AO_fetch_compare_and_swap_emulation(volatile AO_t *addr, AO_t old_val, AO_t
  * which the other atomic operation macros are used... */
 #define MVM_store(addr, new) AO_store_full((volatile AO_t *)(addr), (AO_t)(new))
 #define MVM_load(addr) AO_load_full((volatile AO_t *)(addr))
+
+#endif
