@@ -112,6 +112,9 @@ static MVMuint8 cmp_prim_spec(MVMThreadContext *tc, MVMSpeshStatsType *type_tupl
         if (wanted_prim_spec == MVM_STORAGE_SPEC_BP_INT) {
             return ss->can_box & MVM_STORAGE_SPEC_CAN_BOX_INT;
         }
+        else if (wanted_prim_spec == MVM_STORAGE_SPEC_BP_UINT64) {
+            return ss->can_box & MVM_STORAGE_SPEC_CAN_BOX_INT;
+        }
         else if (wanted_prim_spec == MVM_STORAGE_SPEC_BP_NUM) {
             return ss->can_box & MVM_STORAGE_SPEC_CAN_BOX_NUM;
         }
@@ -154,6 +157,12 @@ static void insert_getarg_and_box(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpe
                 hlltype_op = MVM_OP_hllboxtype_i;
                 fetch_op = MVM_OP_sp_getarg_i;
                 break;
+            case MVM_CALLSITE_ARG_UINT:
+                unboxed_temp = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_uint64);
+                box_op = MVM_OP_box_u;
+                hlltype_op = MVM_OP_hllboxtype_i;
+                fetch_op = MVM_OP_sp_getarg_i;
+                break;
             case MVM_CALLSITE_ARG_NUM:
                 unboxed_temp = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_num64);
                 box_op = MVM_OP_box_n;
@@ -340,7 +349,7 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                     goto cleanup;
                 if (pos_ins[idx]) { /* Dupe; weird. */
                     MVM_spesh_graph_add_comment(tc, g, ins,
-                            "bailed argument spesh: duplicate access to argument %ld", idx);
+                            "bailed argument spesh: duplicate access to argument %d", idx);
                     goto cleanup;
                 }
                 pos_ins[idx] = ins;
@@ -360,7 +369,7 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                     goto cleanup;
                 if (pos_ins[idx]) { /* Dupe; weird. */
                     MVM_spesh_graph_add_comment(tc, g, ins,
-                            "bailed argument spesh: duplicate access to argument %ld", idx);
+                            "bailed argument spesh: duplicate access to argument %d", idx);
                     goto cleanup;
                 }
                 pos_ins[idx] = ins;
@@ -461,17 +470,17 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                         if (arg_type != MVM_CALLSITE_ARG_OBJ ||
                                 !cmp_prim_spec(tc, type_tuple, i, MVM_STORAGE_SPEC_BP_INT)) {
                             MVM_spesh_graph_add_comment(tc, g, pos_ins[i],
-                                    "bailed argument spesh: expected arg flag %ld to be int or box an int; type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
+                                    "bailed argument spesh: expected arg flag %d to be int or box an int; type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
                             goto cleanup;
                         }
                     break;
                 case MVM_OP_param_rp_u:
                 case MVM_OP_param_op_u:
-                    if (arg_type != MVM_CALLSITE_ARG_INT)
+                    if (arg_type != MVM_CALLSITE_ARG_UINT)
                         if (arg_type != MVM_CALLSITE_ARG_OBJ ||
-                                !cmp_prim_spec(tc, type_tuple, i, MVM_STORAGE_SPEC_BP_INT)) {
+                                !cmp_prim_spec(tc, type_tuple, i, MVM_STORAGE_SPEC_BP_UINT64)) {
                             MVM_spesh_graph_add_comment(tc, g, pos_ins[i],
-                                    "bailed argument spesh: expected arg flag %ld to be int or box an int; type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
+                                    "bailed argument spesh: expected arg flag %d to be int or box an int; type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
                             goto cleanup;
                         }
                     break;
@@ -481,7 +490,7 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                         if (arg_type != MVM_CALLSITE_ARG_OBJ ||
                                 !cmp_prim_spec(tc, type_tuple, i, MVM_STORAGE_SPEC_BP_NUM)) {
                             MVM_spesh_graph_add_comment(tc, g, pos_ins[i],
-                                    "bailed argument spesh: expected arg flag %ld to be num or box a num", i);
+                                    "bailed argument spesh: expected arg flag %d to be num or box a num", i);
                             goto cleanup;
                         }
                     break;
@@ -491,16 +500,17 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                         if (arg_type != MVM_CALLSITE_ARG_OBJ ||
                                 !cmp_prim_spec(tc, type_tuple, i, MVM_STORAGE_SPEC_BP_STR)) {
                             MVM_spesh_graph_add_comment(tc, g, pos_ins[i],
-                                    "bailed argument spesh: expected arg flag %ld to be str or box a str type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
+                                    "bailed argument spesh: expected arg flag %d to be str or box a str type at position was %s", i, type_tuple && type_tuple[i].type ? MVM_6model_get_debug_name(tc, type_tuple[i].type) : "null type tuple");
                             goto cleanup;
                         }
                     break;
                 case MVM_OP_param_rp_o:
                 case MVM_OP_param_op_o:
                     if (arg_type != MVM_CALLSITE_ARG_OBJ && arg_type != MVM_CALLSITE_ARG_INT &&
-                        arg_type != MVM_CALLSITE_ARG_NUM && arg_type != MVM_CALLSITE_ARG_STR) {
+                        arg_type != MVM_CALLSITE_ARG_NUM && arg_type != MVM_CALLSITE_ARG_STR &&
+                        arg_type != MVM_CALLSITE_ARG_UINT) {
                         MVM_spesh_graph_add_comment(tc, g, pos_ins[i],
-                                "bailed argument spesh: expected arg flag %ld to be obj or int or num or str", i);
+                                "bailed argument spesh: expected arg flag %d to be obj or int or num or str", i);
                         goto cleanup;
                     }
                     break;
@@ -510,7 +520,7 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
             }
             else if (!pos_ins[i] && !param_sp_ins) {
                 MVM_spesh_graph_add_comment(tc, g, g->entry->first_ins,
-                        "bailed argument spesh: no positional arg fetch op found for %ld", i);
+                        "bailed argument spesh: no positional arg fetch op found for %d", i);
                 goto cleanup;
             }
         }
@@ -648,11 +658,11 @@ void MVM_spesh_args(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCallsite *cs,
                 break;
             case MVM_OP_param_rp_u:
             case MVM_OP_param_op_u:
-                if (arg_type == MVM_CALLSITE_ARG_INT) {
+                if (arg_type == MVM_CALLSITE_ARG_UINT) {
                     pos_ins[i]->info = MVM_op_get_op(MVM_OP_sp_getarg_i);
                 }
                 else {
-                    pos_unbox(tc, g, pos_bb[i], pos_ins[i], MVM_op_get_op(MVM_OP_decont_i));
+                    pos_unbox(tc, g, pos_bb[i], pos_ins[i], MVM_op_get_op(MVM_OP_decont_u));
                     pos_added[i]++;
                     if (type_tuple && type_tuple[i].type)
                         add_facts(tc, g, i, type_tuple[i], pos_ins[i]);
