@@ -37,6 +37,9 @@ void MVM_spesh_disp_initialize_dispatch_op_info(MVMThreadContext *tc,
         else if (flag & MVM_CALLSITE_ARG_INT) {
             dispatch_info->operands[operand_index] = MVM_operand_int64;
         }
+        else if (flag & MVM_CALLSITE_ARG_UINT) {
+            dispatch_info->operands[operand_index] = MVM_operand_uint64;
+        }
         else if (flag & MVM_CALLSITE_ARG_NUM) {
             dispatch_info->operands[operand_index] = MVM_operand_num64;
         }
@@ -445,6 +448,8 @@ MVMOpInfo * MVM_spesh_disp_initialize_resumption_op_info(MVMThreadContext *tc,
                 res_info->operands[operand_index] = MVM_operand_obj;
             else if (flag & MVM_CALLSITE_ARG_INT)
                 res_info->operands[operand_index] = MVM_operand_int64;
+            else if (flag & MVM_CALLSITE_ARG_UINT)
+                res_info->operands[operand_index] = MVM_operand_uint64;
             else if (flag & MVM_CALLSITE_ARG_NUM)
                 res_info->operands[operand_index] = MVM_operand_num64;
             else if (flag & MVM_CALLSITE_ARG_STR)
@@ -463,7 +468,7 @@ MVMOpInfo * MVM_spesh_disp_initialize_resumption_op_info(MVMThreadContext *tc,
 static void insert_resume_inits(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb,
         MVMSpeshIns **insert_after, MVMDispProgram *dp, MVMSpeshOperand *orig_args,
         MVMSpeshOperand *temporaries, MVMint32 deopt_idx) {
-    MVMuint16 i;
+    MVMuint32 i;
     for (i = 0; i < dp->num_resumptions; i++) {
         /* Allocate the instruction. */
         MVMDispProgramResumption *dpr = &(dp->resumptions[i]);
@@ -1188,6 +1193,9 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                 MVM_spesh_usages_add_by_reg(tc, g, rb_ins->operands[cur_op], rb_ins);
                 cur_op++;
 
+                int has_rw_dummy = 0;
+                MVMSpeshOperand rw_dummy = { 0 };
+
                 MVMuint8 *is_rw_operand = MVM_spesh_alloc(tc, g, rb_op->num_operands * sizeof(MVMuint8));
                 MVMSpeshOperand *rw_operands = MVM_spesh_alloc(tc, g, rb_op->num_operands * sizeof(MVMSpeshOperand));
                 if (body) {
@@ -1198,15 +1206,27 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                             switch (arg_types[j - 1] & MVM_NATIVECALL_ARG_TYPE_MASK) {
                                 case MVM_NATIVECALL_ARG_INT:
                                 case MVM_NATIVECALL_ARG_LONG:
-                                case MVM_NATIVECALL_ARG_LONGLONG:
+                                case MVM_NATIVECALL_ARG_LONGLONG: {
+                                    MVMSpeshOperand var = skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j];
                                     is_rw_operand[j] = 1;
                                     rw_operands[j] = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_int64);
-                                    emit_bi_op(tc, g, bb, &insert_after, MVM_OP_decont_i, rw_operands[j], skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j]);
+                                    emit_bi_op(tc, g, bb, &insert_after, MVM_OP_decont_i, rw_operands[j], var);
 
                                     callsite = MVM_callsite_drop_positional(tc, callsite, j);
                                     callsite = MVM_callsite_insert_positional(tc, callsite, j, MVM_CALLSITE_ARG_INT);
                                     MVM_callsite_intern(tc, &callsite, 1, 0);
+
+                                    if (has_return_value && var.reg.orig == rb_ins->operands[0].reg.orig) {
+                                        rw_dummy = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_obj);
+                                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_set, rw_dummy, var);
+                                        has_rw_dummy = 1;
+                                    }
                                     break;
+                                }
+                                default:
+                                    MVM_spesh_graph_add_comment(tc, g, ins, "dispatch not compiled: unsupported NativeCall rw type %d",
+                                                    arg_types[j - 1] & MVM_NATIVECALL_ARG_TYPE_MASK);
+                                    return 0;
                             }
                         }
                     }
@@ -1246,10 +1266,15 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                 MVMuint16 j;
                 for (j = 1; j < callsite->flag_count; j++) {
                     if (is_rw_operand[j]) {
+                        MVMSpeshOperand var = has_rw_dummy
+                            ? rw_dummy
+                            : skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j];
                         emit_bi_op(tc, g, bb->linear_next, &post_call_instructions, MVM_OP_assign_i,
-                            skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j],
-                            rw_operands[j]);
+                            var, rw_operands[j]);
+                        MVM_spesh_usages_add_by_reg(tc, g, var, post_call_instructions);
                         MVM_spesh_manipulate_release_temp_reg(tc, g, rw_operands[j]);
+                        if (has_rw_dummy)
+                            MVM_spesh_manipulate_release_temp_reg(tc, g, var);
                     }
                 }
 
