@@ -3,6 +3,7 @@
 typedef struct MVMActiveHandler MVMActiveHandler;
 typedef struct MVMArgInfo MVMArgInfo;
 typedef struct MVMArgProcContext MVMArgProcContext;
+typedef struct MVMArgs MVMArgs;
 typedef struct MVMArray MVMArray;
 typedef struct MVMArrayBody MVMArrayBody;
 typedef struct MVMArrayREPRData MVMArrayREPRData;
@@ -14,11 +15,27 @@ typedef struct MVMBoolificationSpec MVMBoolificationSpec;
 typedef struct MVMBootTypes MVMBootTypes;
 typedef struct MVMEventSubscriptions MVMEventSubscriptions;
 typedef struct MVMBytecodeAnnotation MVMBytecodeAnnotation;
-typedef struct MVMCallCapture MVMCallCapture;
-typedef struct MVMCallCaptureBody MVMCallCaptureBody;
+typedef struct MVMCapture MVMCapture;
+typedef struct MVMCaptureBody MVMCaptureBody;
 typedef struct MVMCallsite MVMCallsite;
 typedef struct MVMCallsiteInterns MVMCallsiteInterns;
+typedef struct MVMCallStackRecord MVMCallStackRecord;
+typedef struct MVMCallStackStart MVMCallStackStart;
+typedef struct MVMCallStackRegionStart MVMCallStackRegionStart;
+typedef struct MVMCallStackFrame MVMCallStackFrame;
+typedef struct MVMCallStackHeapFrame MVMCallStackHeapFrame;
+typedef struct MVMCallStackPromotedFrame MVMCallStackPromotedFrame;
+typedef struct MVMCallStackContinuationTag MVMCallStackContinuationTag;
+typedef struct MVMCallStackFlattening MVMCallStackFlattening;
+typedef struct MVMCallStackDispatchRecord MVMCallStackDispatchRecord;
+typedef struct MVMCallStackDispatchRun MVMCallStackDispatchRun;
+typedef struct MVMCallStackBindControl MVMCallStackBindControl;
+typedef struct MVMCallStackArgsFromC MVMCallStackArgsFromC;
+typedef struct MVMCallStackDeoptedResumeInit MVMCallStackDeoptedResumeInit;
+typedef struct MVMCallStackNestedRunloop MVMCallStackNestedRunloop;
+typedef struct MVMCallStackSpecialReturn MVMCallStackSpecialReturn;
 typedef struct MVMCallStackRegion MVMCallStackRegion;
+typedef struct MVMCallStackIterator MVMCallStackIterator;
 typedef struct MVMCFunction MVMCFunction;
 typedef struct MVMCFunctionBody MVMCFunctionBody;
 typedef struct MVMCode MVMCode;
@@ -29,10 +46,8 @@ typedef struct MVMCompUnitBody MVMCompUnitBody;
 typedef struct MVMConcatState MVMConcatState;
 typedef struct MVMContainerConfigurer MVMContainerConfigurer;
 typedef struct MVMContainerSpec MVMContainerSpec;
-typedef struct MVMContainerRegistry MVMContainerRegistry;
 typedef struct MVMContext MVMContext;
 typedef struct MVMContextBody MVMContextBody;
-typedef struct MVMContinuationTag MVMContinuationTag;
 typedef struct MVMDecoder MVMDecoder;
 typedef struct MVMDecoderBody MVMDecoderBody;
 typedef struct MVMDLLRegistry MVMDLLRegistry;
@@ -66,7 +81,6 @@ typedef struct MVMHashEntry MVMHashEntry;
 typedef struct MVMHLLConfig MVMHLLConfig;
 typedef struct MVMIntConstCache MVMIntConstCache;
 typedef struct MVMInstance MVMInstance;
-typedef struct MVMInvocationSpec MVMInvocationSpec;
 typedef struct MVMIter MVMIter;
 typedef struct MVMIterBody MVMIterBody;
 typedef struct MVMKnowHOWAttributeREPR MVMKnowHOWAttributeREPR;
@@ -107,9 +121,6 @@ typedef struct MVMCUnion MVMCUnion;
 typedef struct MVMCUnionBody MVMCUnionBody;
 typedef struct MVMCUnionNameMap MVMCUnionNameMap;
 typedef struct MVMCUnionREPRData MVMCUnionREPRData;
-typedef struct MVMMultiCache MVMMultiCache;
-typedef struct MVMMultiCacheBody MVMMultiCacheBody;
-typedef struct MVMMultiCacheNode MVMMultiCacheNode;
 typedef struct MVMMultiDimArray MVMMultiDimArray;
 typedef struct MVMMultiDimArrayBody MVMMultiDimArrayBody;
 typedef struct MVMMultiDimArrayREPRData MVMMultiDimArrayREPRData;
@@ -166,18 +177,15 @@ typedef struct MVMSpeshAnn MVMSpeshAnn;
 typedef struct MVMSpeshFacts MVMSpeshFacts;
 typedef struct MVMSpeshCode MVMSpeshCode;
 typedef struct MVMSpeshCandidate MVMSpeshCandidate;
+typedef struct MVMSpeshCandidateBody MVMSpeshCandidateBody;
 typedef struct MVMSpeshLogGuard MVMSpeshLogGuard;
 typedef struct MVMSpeshCallInfo MVMSpeshCallInfo;
 typedef struct MVMSpeshInline MVMSpeshInline;
 typedef struct MVMSpeshIterator MVMSpeshIterator;
+typedef struct MVMSpeshResumeInit MVMSpeshResumeInit;
 typedef struct MVMSpeshLog MVMSpeshLog;
 typedef struct MVMSpeshLogBody MVMSpeshLogBody;
 typedef struct MVMSpeshLogEntry MVMSpeshLogEntry;
-typedef struct MVMSpeshPluginState MVMSpeshPluginState;
-typedef struct MVMSpeshPluginStateBody MVMSpeshPluginStateBody;
-typedef struct MVMSpeshPluginPosition MVMSpeshPluginPosition;
-typedef struct MVMSpeshPluginGuardSet MVMSpeshPluginGuardSet;
-typedef struct MVMSpeshPluginGuard MVMSpeshPluginGuard;
 typedef struct MVMSpeshStats MVMSpeshStats;
 typedef struct MVMSpeshStatsByCallsite MVMSpeshStatsByCallsite;
 typedef struct MVMSpeshStatsByType MVMSpeshStatsByType;
@@ -186,7 +194,7 @@ typedef struct MVMSpeshStatsByOffset MVMSpeshStatsByOffset;
 typedef struct MVMSpeshStatsTypeCount MVMSpeshStatsTypeCount;
 typedef struct MVMSpeshStatsInvokeCount MVMSpeshStatsInvokeCount;
 typedef struct MVMSpeshStatsTypeTupleCount MVMSpeshStatsTypeTupleCount;
-typedef struct MVMSpeshStatsPluginGuardCount MVMSpeshStatsPluginGuardCount;
+typedef struct MVMSpeshStatsDispatchResultCount MVMSpeshStatsDispatchResultCount;
 typedef struct MVMSpeshStatsStatic MVMSpeshStatsStatic;
 typedef struct MVMSpeshSimStack MVMSpeshSimStack;
 typedef struct MVMSpeshSimStackFrame MVMSpeshSimStackFrame;
@@ -258,6 +266,11 @@ typedef struct MVMJitJumpList MVMJitJumpList;
 typedef struct MVMJitControl MVMJitControl;
 typedef struct MVMJitData MVMJitData;
 typedef struct MVMJitStackSlot MVMJitStackSlot;
+typedef struct MVMJitRunByteCode MVMJitRunByteCode;
+typedef struct MVMJitRunCCode MVMJitRunCCode;
+typedef struct MVMJitRunNativeCall MVMJitRunNativeCall;
+typedef struct MVMJitDispatch MVMJitDispatch;
+typedef struct MVMJitIsType MVMJitIsType;
 typedef struct MVMJitCode MVMJitCode;
 typedef struct MVMJitCompiler MVMJitCompiler;
 typedef struct MVMJitExprTree MVMJitExprTree;
@@ -290,6 +303,7 @@ typedef struct MVMDebugServerBreakpointFileTable MVMDebugServerBreakpointFileTab
 typedef struct MVMDebugServerBreakpointTable MVMDebugServerBreakpointTable;
 typedef struct MVMDebugServerHandleTable MVMDebugServerHandleTable;
 typedef struct MVMDebugServerHandleTableEntry MVMDebugServerHandleTableEntry;
+typedef struct MVMDebugServerRequestData MVMDebugServerRequestData;
 typedef struct MVMDebugServerRequestEntry MVMDebugServerRequestEntry;
 typedef struct MVMDebugServerData MVMDebugServerData;
 typedef struct MVMRunloopState MVMRunloopState;
@@ -298,3 +312,30 @@ typedef struct MVMFixKeyHashTable MVMFixKeyHashTable;
 typedef struct MVMIndexHashTable MVMIndexHashTable;
 typedef struct MVMPtrHashTable MVMPtrHashTable;
 typedef struct MVMUniHashTable MVMUniHashTable;
+typedef struct MVMDispDefinition MVMDispDefinition;
+typedef struct MVMDispRegistry MVMDispRegistry;
+typedef struct MVMDispRegistryTable MVMDispRegistryTable;
+typedef struct MVMDispInlineCache MVMDispInlineCache;
+typedef struct MVMDispInlineCacheEntry MVMDispInlineCacheEntry;
+typedef struct MVMDispInlineCacheEntryResolvedGetLexStatic MVMDispInlineCacheEntryResolvedGetLexStatic;
+typedef struct MVMDispInlineCacheEntryMonomorphicDispatch MVMDispInlineCacheEntryMonomorphicDispatch;
+typedef struct MVMDispInlineCacheEntryMonomorphicDispatchFlattening MVMDispInlineCacheEntryMonomorphicDispatchFlattening;
+typedef struct MVMDispInlineCacheEntryPolymorphicDispatch MVMDispInlineCacheEntryPolymorphicDispatch;
+typedef struct MVMDispInlineCacheEntryPolymorphicDispatchFlattening MVMDispInlineCacheEntryPolymorphicDispatchFlattening;
+typedef struct MVMDispProgramOutcome MVMDispProgramOutcome;
+typedef struct MVMDispProgramRecording MVMDispProgramRecording;
+typedef struct MVMDispProgramRecordingValue MVMDispProgramRecordingValue;
+typedef struct MVMDispProgramRecordingCapture MVMDispProgramRecordingCapture;
+typedef struct MVMDispProgramRecordingResumeInit MVMDispProgramRecordingResumeInit;
+typedef struct MVMDispProgramRecordingResumption MVMDispProgramRecordingResumption;
+typedef struct MVMDispProgram MVMDispProgram;
+typedef union MVMDispProgramConstant MVMDispProgramConstant;
+typedef struct MVMDispProgramOp MVMDispProgramOp;
+typedef struct MVMDispProgramResumption MVMDispProgramResumption;
+typedef struct MVMDispProgramResumptionInitValue MVMDispProgramResumptionInitValue;
+typedef struct MVMDispResumptionData MVMDispResumptionData;
+typedef struct MVMDispResumptionState MVMDispResumptionState;
+typedef struct MVMDispSysCall MVMDispSysCall;
+typedef struct MVMDispSysCallHashEntry MVMDispSysCallHashEntry;
+typedef struct MVMTracked MVMTracked;
+typedef struct MVMTrackedBody MVMTrackedBody;
